# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
import pandas as pd

def get_cal_and_trade_cal(start, end):
    """
    获取交易日期
    """
    cal = HBDB().read_cal(start, end)
    cal = cal.rename(columns={'JYRQ': 'TRADE_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
    cal['IS_OPEN'] = cal['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
    cal['IS_WEEK_END'] = cal['IS_WEEK_END'].fillna(0).astype(int)
    cal['IS_MONTH_END'] = cal['IS_MONTH_END'].fillna(0).astype(int)
    cal = cal.sort_values('TRADE_DATE')
    trade_cal = cal[cal['IS_OPEN'] == 1]
    trade_cal['RECENT_TRADE_DATE'] = trade_cal['TRADE_DATE']
    trade_cal['PREV_TRADE_DATE'] = trade_cal['TRADE_DATE'].shift(1)
    trade_cal = trade_cal[['TRADE_DATE', 'RECENT_TRADE_DATE', 'PREV_TRADE_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END']]
    cal = cal.merge(trade_cal[['TRADE_DATE', 'RECENT_TRADE_DATE']], on=['TRADE_DATE'], how='left')
    cal['RECENT_TRADE_DATE'] = cal['RECENT_TRADE_DATE'].fillna(method='ffill')
    cal = cal.merge(trade_cal[['TRADE_DATE', 'PREV_TRADE_DATE']], on=['TRADE_DATE'], how='left')
    cal['PREV_TRADE_DATE'] = cal['PREV_TRADE_DATE'].fillna(method='bfill')
    cal = cal[['TRADE_DATE', 'RECENT_TRADE_DATE', 'PREV_TRADE_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END']]
    return cal, trade_cal

# company_list = ['阿里巴巴-SW', '京东集团-SW', '凯莱英', '美团-W', '腾讯控股']
# holding_list = []
# for company in company_list:
#     holding = pd.read_excel('C:/Users/wenqian.xu/Desktop/JGCG/{}机构持股.xlsx'.format(company), sheet_name='file', index_col=0).reset_index().drop('index', axis=1)
#     holding = holding.dropna(subset=['机构名称'])
#     holding = holding[['类型', '持仓市值']].groupby(['类型']).sum().reset_index()
#     holding['持仓市值占比'] = holding['持仓市值'] / holding['持仓市值'].sum()
#     holding['公司'] = company
#     holding_list.append(holding)
# holding = pd.concat(holding_list)
# holding = holding[['类型', '持仓市值占比']].groupby(['类型']).mean().reset_index()
# holding.to_excel('C:/Users/wenqian.xu/Desktop/JGCG/机构持股.xlsx')

# holding_change_total = get_cal_and_trade_cal('20210315', '20220315')[0][['TRADE_DATE']].rename(columns={'TRADE_DATE': '变动日期'})
# company_list = ['阿里巴巴-SW', '京东集团-SW', '凯莱英', '美团-W', '腾讯控股']
# holding_change_list = []
# for company in company_list:
#     holding_change = pd.read_excel('C:/Users/wenqian.xu/Desktop/JGCG/{}异动分析.xlsx'.format(company), sheet_name='file', index_col=0).reset_index().drop('Unnamed: 1', axis=1)
#     cg_change = holding_change[['变动日期', '类型', '持股变动(股)']].groupby(['变动日期', '类型']).sum().reset_index().rename(columns={'持股变动(股)': '持股变动(股)_{}'.format(company)})
#     sz_change = holding_change[['变动日期', '类型', '市值变动']].groupby(['变动日期', '类型']).sum().reset_index().rename(columns={'市值变动': '市值变动_{}'.format(company)})
#     holding_change = cg_change.merge(sz_change, on=['变动日期', '类型'], how='left')
#     holding_change = holding_change[holding_change['类型'] == '国际'].drop('类型', axis=1)
#     holding_change['变动日期'] = holding_change['变动日期'].apply(lambda x: x.replace('/', ''))
#     holding_change_total = holding_change_total.merge(holding_change, on=['变动日期'], how='left')
# holding_change_total.to_excel('C:/Users/wenqian.xu/Desktop/JGCG/异动分析.xlsx')

# holding_change_total = get_cal_and_trade_cal('20210315', '20220315')[0][['TRADE_DATE']].rename(columns={'TRADE_DATE': '变动日期'})
# company_list = ['阿里巴巴-SW', '京东集团-SW', '凯莱英', '美团-W', '腾讯控股']
# holding_change_list = []
# for company in company_list:
#     holding_change = pd.read_excel('C:/Users/wenqian.xu/Desktop/JGCG/{}异动分析.xlsx'.format(company), sheet_name='file', index_col=0).reset_index().drop('Unnamed: 1', axis=1)
#     holding_change = holding_change[['变动日期', '类型', '变动占总股数比']].groupby(['变动日期', '类型']).sum().reset_index().rename(columns={'变动占总股数比': '变动占总股数比_{}'.format(company)})
#     holding_change = holding_change[holding_change['类型'] == '国际'].drop('类型', axis=1)
#     holding_change['变动日期'] = holding_change['变动日期'].apply(lambda x: x.replace('/', ''))
#     holding_change_total = holding_change_total.merge(holding_change, on=['变动日期'], how='left')
# holding_change_total.to_excel('C:/Users/wenqian.xu/Desktop/JGCG/异动分析.xlsx')


import pandas as pd
from WindPy import w
from datetime import datetime

w.start()

valuation_df = pd.read_csv('C:\\Users\\kai.zhang\\Desktop\\可转债\\可转债估值结果.csv')
cb_id_list = valuation_df['cb_id'].tolist()
res = w.wsd(','.join(cb_id_list), "close", "2021-06-01", "2021-06-08", "")

data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
data.index.name = 'trade_date'
data.reset_index(inplace=True)
data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
data = data.set_index('trade_date')

period_ret = data.pct_change(periods=5).iloc[-1]
valuation_df = pd.merge(valuation_df, period_ret.to_frame('period_ret'), left_on='cb_id', right_index=True)
valuation_df['deviation_bs'] = (valuation_df['close'] - valuation_df['B-S']) / valuation_df['B-S']
valuation_df['deviation_tree'] = (valuation_df['close'] - valuation_df['tree']) / valuation_df['tree']
valuation_df['deviation_mc'] = (valuation_df['close'] - valuation_df['mc']) / valuation_df['mc']
n = valuation_df.shape[0] // 10
valuation_df = valuation_df.sort_values(by='deviation_bs').reset_index(drop=True)
for i in range(10):
    valuation_df.loc[i * n:(i + 1) * n, 'sign'] = i
valuation_df.loc[(i + 1) * n:, 'sign'] = 9
a = valuation_df.groupby('sign')['period_ret'].mean()

valuation_df = valuation_df.sort_values(by='deviation_tree').reset_index(drop=True)
for i in range(10):
    valuation_df.loc[i * n:(i + 1) * n, 'sign'] = i
valuation_df.loc[(i + 1) * n:, 'sign'] = 9
b = valuation_df.groupby('sign')['period_ret'].mean()

valuation_df = valuation_df.sort_values(by='deviation_mc').reset_index(drop=True)
for i in range(10):
    valuation_df.loc[i * n:(i + 1) * n, 'sign'] = i
valuation_df.loc[(i + 1) * n:, 'sign'] = 9
c = valuation_df.groupby('sign')['period_ret'].mean()

import matplotlib.pyplot as plt
plt.style.use('seaborn')
df = pd.concat([a, b, c], axis=1)
df.columns = ['B-S', 'Tree', 'MC']
df = df.reset_index()
df['sign'] = df['sign'].apply(lambda x: "Group_{}".format(str(int(x + 1))))

df.set_index('sign').plot.bar()


import pymysql
import numpy as np
import pandas as pd
from scipy import optimize as sco


def generate_table(database, table, generate_sql, sql_ip, sql_user, sql_pass, table_comment=''):
    db = pymysql.connect(host=sql_ip, user=sql_user, password=sql_pass, database=database)

    cursor = db.cursor()

    sql = 'create table if not exists `' + table + '` ' + generate_sql + ' comment=\'' + table_comment + '\''
    cursor.execute(sql)
    db.close()


def portfolio_var(weights, cov_matrix):
    return np.dot(weights.T, np.dot(cov_matrix, weights))


def risk_contribute(weights, cov_matrix):
    std = np.sqrt(portfolio_var(weights, cov_matrix))
    mrc = np.matrix(cov_matrix) * np.matrix(weights).T
    return np.multiply(mrc, np.matrix(weights).T) / std


def portfolio_annualised_performance(weights, mean_returns, cov_matrix, freq=252):
    returns = np.sum(mean_returns * weights) * freq
    std = np.sqrt(portfolio_var(weights, cov_matrix)) * np.sqrt(freq)
    return std, returns


def negative_sharpe_ratio(weights, mean_returns, cov_matrix, risk_free_rate, freq=252):
    p_var, p_ret = portfolio_annualised_performance(weights, mean_returns, cov_matrix, freq=freq)
    return -(p_ret - risk_free_rate) / p_var


def max_sharpe_ratio(mean_returns, cov_matrix, risk_free_rate=0.015, freq=252, single_max=0.3):
    num_assets = len(mean_returns)
    args = (mean_returns, cov_matrix, risk_free_rate, freq)
    constraints = ({'type': 'eq', 'fun': lambda x: np.sum(x) - 1})
    # bound = (0.0, 1.0)
    # bounds = tuple(bound for asset in range(num_assets))
    bounds = sco.Bounds([0] * num_assets, [single_max] * num_assets)
    result = sco.minimize(
        negative_sharpe_ratio,
        np.array(num_assets * [1 / num_assets]),
        args=args,
        method='SLSQP',
        bounds=bounds,
        constraints=constraints
    )
    return pd.DataFrame(
        result.x,
        index=mean_returns.index,
        columns=['allocation']
    )



import pandas as pd
import numpy as np
#ploting
import matplotlib.pyplot as plt

# Modeling
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import RobustScaler


# Metrics validation
from sklearn.metrics import confusion_matrix, \
    precision_score, recall_score, f1_score, \
    confusion_matrix, precision_recall_curve, \
    accuracy_score

def set_metrics_method(y_pred,y_test):

    if(len(set(y_test))>2):
        method='weighted'
    else:
        method='binary'
    return method

def metrics(actuals, predictions,method):
    print("Accuracy: {:.5f}".format(accuracy_score(actuals.tolist(), predictions.tolist())))
    print("Precision: {:.5f}".format(precision_score(actuals.tolist(), predictions.tolist(),average=method)))
    print("Recall: {:.5f}".format(recall_score(actuals.tolist(), predictions.tolist(),average=method)))
    print("F1-score: {:.5f}".format(f1_score(actuals.tolist(), predictions.tolist(),average=method)))
    return f1_score(actuals.tolist(), predictions.tolist(),average=method)

def train_model (model, x_train, y_train):

    model.fit(x_train, y_train)

    return model

def evluation(model,x_test,y_test):

    y_pred = model.predict(x_test)

    if (type(y_pred[0]) == np.float32 or type(y_pred[0]) == np.float64):
        y_pred = y_pred.round().astype(int)

    method = set_metrics_method(y_pred,y_test)
    f1_score = metrics(y_test, y_pred, method)
    #cnf_matrix = confusion_matrix(y_test, y_pred)

    # sns.heatmap(pd.DataFrame(cnf_matrix), annot=True, cmap="YlGnBu", fmt='g')
    # plt.ylabel('Actual Label')
    # plt.xlabel('Predicted Label')

    return f1_score

def load_local_data(dir):

    df = pd.read_csv(dir)
    return  df

def standardlize(df,col_list):
    # rs_list=[]
    rs = RobustScaler()
    for col in col_list:
        # tempfit=rs.fit((df[col].values.reshape(-1,1)))
        # df[col] = tempfit.transform((df[col].values.reshape(-1, 1)))
        df[col]=rs.fit_transform ((df[col].values.reshape(-1,1)))
        # rs_list.append(tempfit)
        #dict(zip(col_list,rs_list))
    return df

def split_train_test(x,y,test_ratio):
    (x_train, x_test, y_train, y_test) = train_test_split(x, y, test_size= test_ratio, random_state= 42)
    # print("Shape of train_X: ", x_train.shape)
    # print("Shape of test_X: ", x_test.shape)
    return x_train, x_test, y_train, y_test

def model_built_up(df,lable_col,model_name,standardlize_col,test_ratio):

    df=standardlize(df,standardlize_col)

    x = df.drop([lable_col], axis= 1)
    y = df[lable_col]

    if(test_ratio>0):
        x_train, x_test, y_train, y_test=split_train_test(x,y,test_ratio)
    else:
        x_train=x
        y_train=y

    if(model_name.lower()=='xgboost'):

        map_list=y_train.unique()
        map_list.sort()
        for i in range(len(map_list)):
            y_train[y_train==map_list[i]]=i
            if(test_ratio>0):
                y_test[y_test == map_list[i]] = i
        from xgboost import XGBRegressor
        if(len(map_list)>2):
            trained_model=train_model(XGBRegressor(objective='multi:softmax',num_class=len(map_list)), x_train, y_train)
        else:
            trained_model = train_model(XGBRegressor(), x_train, y_train)
    elif(model_name.lower()=='randomforest'):
        from sklearn.ensemble import RandomForestClassifier
        trained_model=train_model(RandomForestClassifier(), x_train, y_train)
    elif(model_name.lower()=='svm'):
        from sklearn.svm import SVC
        trained_model = train_model(SVC(), x_train, y_train)
    elif(model_name.lower()=='kmeans'):
        from sklearn.cluster import KMeans
        n_clusters=len(set(y))
        trained_model=train_model(KMeans(n_clusters=n_clusters), x_train, x_test, y_test)
        f1_score=0
    else:
        raise Exception
        print('the input model name is not acceptable,')

    if(test_ratio>0 and model_name.lower()!='kmeans'):
        print('Evaluation of {0}'.format(model_name))
        f1_score=evluation(trained_model,x_test,y_test)
    else:
        f1_score=0

    return trained_model,f1_score

if __name__ == '__main__':

    df=load_local_data('creditcard.csv')
    model, f1_score=model_built_up(df,'Class','randomforest',['Amount','Time'],0.2)


# -*- coding:utf-8 -*-

"""
获取基金净值数据接口
Created on 2020/6/15
@author: Meng Lv
@group: Howbuy FDC
@contact: meng.lv@howbuy.com
"""

from __future__ import division
import time
import pandas as pd
from hbshare.db.fund import cons as ct
import hbshare as hbs


def get_authorized_product_list(access_token,include_org='false',product_type='200', retry_count=3, pause=0.01, timeout=30):
    """
        获得公募基金最新净值（含最新回报，区间回报）
    :param access_token: 数据访问令牌
    :param include_org: 是否包含产品所属机构下所有产品
    :param product_type:产品类别
    :return:
    """
    api = hbs.hb_api()
    for _ in range(retry_count):
        time.sleep(pause)
        ct._write_console()
        url = ct.HOWBUY_RD_PROFUDUCT_PERMOISSIONS % (ct.P_TYPE['https'], ct.DOMAINS['ams'],
                                                     access_token,product_type,include_org)
        print(url)
        resp = api.query(url)
        status_code = resp['code']
        if status_code != '0000':
            status = str(resp['desc'] + ": 请确定是否有权限查询！")
            raise ValueError(status)

        if 'body' not in resp:
            status = "未查询到数据"
            raise ValueError(status)

        data = resp['body']
        fund_df = pd.DataFrame(data, columns=ct.HOWBUY_RD_PROFUDUCT_COLUMNS)

        fund_df['code'] = fund_df['code'].astype(str)
        fund_df['name'] = fund_df['name'].astype(str)
        fund_df['fullName'] = fund_df['fullName'].astype(str)
        fund_df['type'] = fund_df['type'].astype(str)
        fund_df['typeName'] = fund_df['typeName'].astype(str)

        return fund_df
    raise IOError(ct.NETWORK_URL_ERROR_MSG)




# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
from datetime import datetime
import matplotlib.pyplot as plt
import pandas as pd
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False


def get_cal_and_trade_cal(start, end):
    """
    获取交易日期
    """
    cal = HBDB().read_cal(start, end)
    cal = cal.rename(columns={'JYRQ': 'TRADE_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
    cal['IS_OPEN'] = cal['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
    cal['IS_WEEK_END'] = cal['IS_WEEK_END'].fillna(0).astype(int)
    cal['IS_MONTH_END'] = cal['IS_MONTH_END'].fillna(0).astype(int)
    cal = cal.sort_values('TRADE_DATE')
    trade_cal = cal[cal['IS_OPEN'] == 1]
    trade_cal['RECENT_TRADE_DATE'] = trade_cal['TRADE_DATE']
    trade_cal['PREV_TRADE_DATE'] = trade_cal['TRADE_DATE'].shift(1)
    trade_cal = trade_cal[['TRADE_DATE', 'RECENT_TRADE_DATE', 'PREV_TRADE_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END']]
    cal = cal.merge(trade_cal[['TRADE_DATE', 'RECENT_TRADE_DATE']], on=['TRADE_DATE'], how='left')
    cal['RECENT_TRADE_DATE'] = cal['RECENT_TRADE_DATE'].fillna(method='ffill')
    cal = cal.merge(trade_cal[['TRADE_DATE', 'PREV_TRADE_DATE']], on=['TRADE_DATE'], how='left')
    cal['PREV_TRADE_DATE'] = cal['PREV_TRADE_DATE'].fillna(method='bfill')
    cal = cal[['TRADE_DATE', 'RECENT_TRADE_DATE', 'PREV_TRADE_DATE', 'IS_OPEN', 'IS_WEEK_END', 'IS_MONTH_END']]
    return cal, trade_cal

def get_industry_stock():
    # stock_industry = HBDB().read_stock_industry()
    # stock_industry.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/stock_industry.hdf', key='table', mode='w')
    stock_industry = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/stock_industry.hdf', key='table')
    stock_industry = stock_industry.rename(columns={'zqdm': 'TICKER_SYMBOL', 'flmc': 'INDUSTRY_NAME', 'fldm': 'INDUSTRY_ID', 'fljb': 'INDUSTRY_TYPE', 'hyhfbz': 'INDUSTRY_VERSION', 'qsrq': 'BEGIN_DATE', 'jsrq': 'END_DATE', 'sfyx': 'IS_NEW'})
    stock_industry['END_DATE'] = stock_industry['END_DATE'].fillna(20990101)
    stock_industry['BEGIN_DATE'] = stock_industry['BEGIN_DATE'].astype(str)
    stock_industry['END_DATE'] = stock_industry['END_DATE'].astype(int).astype(str)
    stock_industry['INDUSTRY_VERSION'] = stock_industry['INDUSTRY_VERSION'].astype(int)
    stock_industry['INDUSTRY_TYPE'] = stock_industry['INDUSTRY_TYPE'].astype(int)
    stock_industry['IS_NEW'] = stock_industry['IS_NEW'].astype(int)
    stock_industry = stock_industry[stock_industry['INDUSTRY_VERSION'] == 2]
    stock_industry = stock_industry[stock_industry['INDUSTRY_TYPE'] == 1]
    stock_industry = stock_industry[['INDUSTRY_NAME', 'TICKER_SYMBOL', 'BEGIN_DATE', 'END_DATE', 'IS_NEW']]
    return stock_industry

def main(index_list, report_date, industry_name):
    report_dates = sorted([i + j for i in [str(int(report_date[:4]) - 2), str(int(report_date[:4]) - 1), str(int(report_date[:4]))] for j in ['0331', '0630', '0930', '1231']])
    report_dates = [date for date in report_dates if date <= report_date][-5:]
    cal = get_cal_and_trade_cal('20050101', report_date)[0]
    report_trade_dates = [cal[cal['TRADE_DATE'] == date]['RECENT_TRADE_DATE'].values[0] for date in report_dates]
    industry_stock = get_industry_stock()
    industry_stock = industry_stock[industry_stock['INDUSTRY_NAME'] == industry_name]
    industry_stock = industry_stock[industry_stock['IS_NEW'] == 1]

    finance_data_list = []
    for date in report_dates:
        print(date)
        finance_data_date = HBDB().read_finance_data_given_date(date)
        finance_data_list.append(finance_data_date)
    finance_data = pd.concat(finance_data_list)
    finance_data.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/finance_data.hdf', key='table', mode='w')
    finance_data = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/finance_data.hdf', key='table')
    finance_data = finance_data.loc[finance_data['TICKER_SYMBOL'].str.len() == 6]
    finance_data = finance_data.loc[finance_data['TICKER_SYMBOL'].astype(str).str.slice(0, 1).isin(['0', '3', '6'])]
    finance_data = finance_data[['END_DATE', 'PUBLISH_DATE', 'TICKER_SYMBOL'] + index_list].rename(columns={'END_DATE': 'REPORT_DATE'})
    finance_data = finance_data.sort_values(['TICKER_SYMBOL', 'REPORT_DATE', 'PUBLISH_DATE']).drop_duplicates(['TICKER_SYMBOL', 'REPORT_DATE'], keep='last')
    finance_data = finance_data.merge(cal[['TRADE_DATE', 'RECENT_TRADE_DATE']].rename(columns={'TRADE_DATE': 'REPORT_DATE'}), on=['REPORT_DATE'], how='left')

    stock_data_list = []
    for date in report_trade_dates:
        print(date)
        stock_data_date = HBDB().read_stock_market_value_given_date(date)
        stock_data_list.append(stock_data_date)
    stock_data = pd.concat(stock_data_list)
    stock_data.to_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/market_value.hdf', key='table', mode='w')
    stock_data = pd.read_hdf('D:/Git/hbshare/hbshare/fe/xwq/data/market_value.hdf', key='table')
    stock_data = stock_data.loc[stock_data['TICKER_SYMBOL'].str.len() == 6]
    stock_data = stock_data.loc[stock_data['TICKER_SYMBOL'].astype(str).str.slice(0, 1).isin(['0', '3', '6'])]
    stock_data = stock_data[['TRADE_DATE', 'TICKER_SYMBOL', 'SEC_SHORT_NAME', 'MARKET_VALUE', 'NEG_MARKET_VALUE']]
    stock_data = finance_data.rename(columns={'RECENT_TRADE_DATE': 'TRADE_DATE'}).merge(stock_data, on=['TRADE_DATE', 'TICKER_SYMBOL'], how='left')
    stock_data = stock_data[stock_data['TICKER_SYMBOL'].isin(industry_stock['TICKER_SYMBOL'].unique().tolist())]
    # stock_data = stock_data.merge(industry_stock[['TICKER_SYMBOL', 'BEGIN_DATE', 'END_DATE']], on=['TICKER_SYMBOL'], how='left')
    # stock_data = stock_data[(stock_data['BEGIN_DATE'] < stock_data['REPORT_DATE']) & (stock_data['END_DATE'] > stock_data['REPORT_DATE'])]

    total_ticker = len(industry_stock[industry_stock['IS_NEW'] == 1])
    use_ticker = len(stock_data[stock_data['REPORT_DATE'] == report_date])

    stock_data_weight = stock_data[['REPORT_DATE', 'MARKET_VALUE']].groupby('REPORT_DATE').sum().reset_index().rename(columns={'MARKET_VALUE': 'TOTAL_MARKET_VALUE'})
    stock_data = stock_data.merge(stock_data_weight, on=['REPORT_DATE'], how='left')
    industry_data_list = []
    for index in index_list:
        stock_data['{}_WEIGHTED'.format(index)] = stock_data[index] * stock_data['MARKET_VALUE'] / stock_data['TOTAL_MARKET_VALUE']
        industry_data = stock_data[['REPORT_DATE', '{}_WEIGHTED'.format(index)]].groupby('REPORT_DATE').sum().rename(columns={'{}_WEIGHTED'.format(index): index})
        industry_data_list.append(industry_data)
    industry_data = pd.concat(industry_data_list, axis=1)
    industry_data[index_list] = industry_data[index_list] / 100.0
    industry_data.index = map(lambda x: datetime.strptime(x, '%Y%m%d'), industry_data.index)
    industry_data['行业名称'] = industry_name
    industry_data = industry_data[['行业名称'] + index_list]

    fig = plt.figure(figsize=(10, 8))
    ax = fig.add_subplot(111)
    for index in index_list:
        ax.plot(industry_data.index, industry_data[index].values, label=index)
    plt.legend()
    plt.title('{0}盈利指标变动（已出报告数/应出报告数：{1}/{2})'.format(industry_name, use_ticker, total_ticker))
    plt.savefig('D:/Git/hbshare/hbshare/fe/xwq/data/industry/{0}.png'.format(industry_name))
    return industry_data


if __name__ == '__main__':
    report_date = '20211231'
    index_list = ['ROE_TTM', 'ROA_TTM', 'ROIC_TTM', 'NET_PROFIT_RATIO_TTM', 'GROSS_INCOME_RATIO_TTM', 'EBIT_ASSET_RATIO_TTM']
    industry_stock = get_industry_stock()
    industry_name_list = industry_stock[industry_stock['IS_NEW'] == 1]['INDUSTRY_NAME'].unique().tolist()
    result_list = []
    for industry_name in industry_name_list:
        result = main(index_list, report_date, industry_name)
        result_list.append(result)
    result = pd.concat(result_list)
    result.to_excel('D:/Git/hbshare/hbshare/fe/xwq/data/industry/data.xlsx')


# -*- coding: utf-8 -*-

from datetime import date, timedelta
import datetime
import matplotlib.dates as mdates
import numpy as np
import pandas as pd
import re
import time


class TimeDateUtil(object):
    @staticmethod
    def convert_str_to_date(src_str, src_format):
        try:
            d = datetime.datetime.strptime(src_str, src_format)
        except ValueError:
            d = None
        except TypeError:
            d = None
        return d

    @staticmethod
    def covert_date_to_str(src_date, tgt_format):
        return src_date.strftime(tgt_format)

    @staticmethod
    def convert_format(src_str, src_format, tgt_format):
        src_date = TimeDateUtil.convert_str_to_date(src_str, src_format)
        if src_date is None:
            return None
        return TimeDateUtil.covert_date_to_str(src_date, tgt_format)

    @staticmethod
    def is_valid_date_format(src_str, src_format):
        return TimeDateUtil.convert_str_to_date(src_str, src_format) is not None

    @staticmethod
    def get_cur_date_str(tgt_format):
        now = datetime.datetime.now()
        return TimeDateUtil.covert_date_to_str(now, tgt_format)

    @staticmethod
    def get_last_days_date(tgt_format, n):
        last_days = date.today() - timedelta(n)
        return last_days.strftime(tgt_format)

    @staticmethod
    def get_previous_date_str(src_str, src_format, tgt_format, n):
        last_days = TimeDateUtil.convert_str_to_date(src_str, src_format) - timedelta(n)
        return last_days.strftime(tgt_format)

    @staticmethod
    def get_current_timestamp():
        return time.time()

    @staticmethod
    def get_timestamp_arr(arr, interpolation=False):
        """
        :param arr: '2016-01-01'
        :param interpolation:
        :return: 736332.0
        """
        if len(arr) == 0:
            return np.empty([0, 0])
        if not interpolation:
            return np.vectorize(lambda dt: mdates.datestr2num(dt))(arr)
        else:
            first_val = mdates.datestr2num(arr[0])
            return np.array([first_val + idx for idx, i in enumerate(arr)])

    @staticmethod
    def get_time_delta(src_str, tgt_str, src_format, tgt_format, unit='DAY'):
        last_days = TimeDateUtil.convert_str_to_date(src_str, src_format) - TimeDateUtil.convert_str_to_date(tgt_str, tgt_format)
        if unit == 'DAY':
            return last_days.days
        elif unit == 'SEC':
            return last_days.seconds
        return last_days.days

    @staticmethod
    def reset_time_format(df, src_format, tgt_format, column_name='TRADE_DATE'):
        """
        转换trade_date格式为“yyyy"
        :return:
        """
        new_time_index = list(
            map(lambda x: TimeDateUtil.convert_format(str(x), src_format, tgt_format), df[column_name].values))
        df[column_name] = pd.Series(new_time_index, index=df.index)
        return df

    @staticmethod
    def check_new_month(prev_date_str, cur_date_str):
        prev_year_month = prev_date_str[:7]
        cur_year_month = cur_date_str[:7]
        return prev_year_month, cur_year_month

    @staticmethod
    def check_regex(src_trade_date, reg):
        mat_reg = re.compile(reg)
        result = mat_reg.match(src_trade_date)
        return result

    @staticmethod
    def extract_year_month_day(date_str, date_format):
        dt = TimeDateUtil.convert_str_to_date(date_str, date_format)
        if dt is None:
            return None, None, None
        return dt.year, dt.month, dt.day

    @staticmethod
    def comp_ttm(fiscal_df,factor_name, date_format):
        """
        考虑有“财报-更正”情况的TTM
        针对利润表、现金流量表计算TTM
        注意：nan值的处理，现金流量表附表频率为半年
        针对资产负债表计算MEAN
        :return:
        """
        def compute_ttm(idxs, df, factor_name, factor_name_quarter):
            part_df = df.iloc[list(map(int, idxs))].dropna(subset=[factor_name_quarter], axis=0)
            if len(part_df.index) == 0:
                return np.nan
            elif part_df['MONTH'].iloc[-1] == 12:
                return part_df[factor_name].iloc[-1]
            elif len(part_df['MONTH'].unique().tolist()) < 4:
                return np.nan
            return np.sum(part_df.drop_duplicates(subset=['YEAR', 'MONTH'], keep='last')[factor_name_quarter])

        fiscal_df['YEAR'] = fiscal_df['END_DATE'].apply(lambda x: TimeDateUtil.convert_str_to_date(x, date_format).year)
        fiscal_df['MONTH'] = fiscal_df['END_DATE'].apply(lambda x: TimeDateUtil.convert_str_to_date(x, date_format).month)
        prev_year, prev_month, prev_val = 0, 0, 0.0
        factor_name_quarter = "{0}_{1}".format(factor_name, 'QUARTER')
        factor_name_ttm = "{0}_{1}".format(factor_name, 'TTM')
        for idx, (_, rows) in enumerate(fiscal_df.itterrows()):
            cur_year, cur_month, __ = TimeDateUtil.extract_year_month_day(rows['END_DATE'], date_format)
            next_year, next_month, __ = TimeDateUtil.extract_year_month_day(
                fiscal_df['END_DATE'].iloc[min(idx+1, len(fiscal_df.index)-1)], date_format)
            if cur_month == 3 and not np.isnan(prev_val):
                fiscal_df.loc[_, factor_name_quarter] = rows[factor_name]
            elif cur_month - prev_month == 3 and prev_year == cur_year and not np.isnan(prev_val):
                quarter_val = rows[factor_name] - prev_val
                fiscal_df.loc[_, factor_name_quarter] = quarter_val
            elif cur_month == prev_month and prev_year == cur_year and not np.isnan(prev_val):
                quarter_val = rows[factor_name] - prev_val
                fiscal_df.loc[_, factor_name_quarter] = quarter_val
            else:
                fiscal_df.loc[_, factor_name_quarter] = np.nan
            # 仅当这种情况下更新之前的数据
            if next_year != cur_year or next_month != cur_month:
                prev_year, prev_month = cur_year, cur_month
                prev_val = fiscal_df.loc[_, factor_name]
        # 滚动计算TTM
        tot = len(fiscal_df.index)
        fiscal_df['IDX'] = range(tot)
        fiscal_df[factor_name_ttm] = fiscal_df['IDX'].rolling(window=tot, min_periods=0, center=False)\
            .apply(lambda x: compute_ttm(x, fiscal_df, factor_name, factor_name_quarter))
        return fiscal_df.drop(['YEAR', 'MONTH', 'IDX', factor_name_quarter], axis=1)

    @staticmethod
    def comp_ma(fiscal_df, factor_name, date_format, window=4):
        """
        注意nan值的处理
        针对资产负债表计算MEAN
        :return:
        """
        def compute_ma(idxs, df, factor_name, window):
            part_df = df.iloc[list(map(int, idxs))]
            if len(part_df.index) == 0:
                return np.nan
            last_publish_date = part_df['PUBLISH_DATE'].iloc[-1]
            last_end_date = part_df['END_DATE'].iloc[-1]
            part_df = part_df[(part_df['END_DATE'] <= last_end_date) & (part_df['PUBLISH_DATE'] <= last_publish_date)] \
                .drop_duplicates(subset=['TICKER_SYMBOL', 'END_DATE'], keep='last') \
                .sort_values(['TICKER_SYMBOL', 'END_DATE', 'PUBLISH_DATE'], ascending=[True, True, True]) \
                .dropna(subset=[factor_name], axis=0)
            return np.mean(part_df.drop_duplicates(subset=['YEAR', 'MONTH'], keep='last').tail(window)[factor_name])

        factor_name_mean = "{0}_{1}".format(factor_name, 'MA')
        fiscal_df['YEAR'] = fiscal_df['END_DATE'].apply(lambda x: TimeDateUtil.convert_str_to_date(x, date_format).year)
        fiscal_df['MONTH'] = fiscal_df['END_DATE'].apply(lambda x: TimeDateUtil.convert_str_to_date(x, date_format).month)
        # 均值
        fiscal_df = fiscal_df.sort_values(by=['TICKER_SYMBOL', 'PUBLISH_DATE', 'END_DATE'], ascending=[True, True, True])
        tot = len(fiscal_df.index)
        fiscal_df['IDX'] = range(tot)
        fiscal_df[factor_name_mean] = fiscal_df['IDX'].rolling(window=tot, min_periods=0, center=False) \
            .apply(lambda x: compute_ma(x, fiscal_df, factor_name, window), raw=True)
        return fiscal_df.drop(['YEAR', 'MONTH', 'IDX'], axis=1)

    @staticmethod
    def comp_diff(fiscal_df, factor_name, date_format):
        """
        注意nan值的处理
        针对资产负债表计算上期变化
        :return:
        """
        def compute_diff(idxs, df, factor_name):
            part_df = df.iloc[list(map(int, idxs))].dropna(subset=[factor_name], axis=0)
            if len(part_df.index) == 0:
                return np.nan
            return part_df.drop_duplicates(subset=['YEAR', 'MONTH'], keep='last')[factor_name].diff(period=1).iloc[-1]

        factor_name_delta = "{0}_{1}".format(factor_name, 'DELTA')
        fiscal_df['YEAR'] = fiscal_df['END_DATE'].apply(lambda x: TimeDateUtil.convert_str_to_date(x, date_format).year)
        fiscal_df['MONTH'] = fiscal_df['END_DATE'].apply(lambda x: TimeDateUtil.convert_str_to_date(x, date_format).month)
        # 与上期之差
        tot = len(fiscal_df.index)
        fiscal_df['IDX'] = range(tot)
        fiscal_df[factor_name_delta] = fiscal_df['IDX'].rolling(window=tot, min_periods=0, center=False) \
            .apply(lambda x: compute_diff(x, fiscal_df, factor_name), raw=True)
        return fiscal_df.drop(['YEAR', 'MONTH', 'IDX'], axis=1)


#!/usr/bin/python
# coding:utf-8
import pandas as pd
import numpy as np
from sqlalchemy import create_engine
from datetime import datetime, timedelta
from hbshare.quant.CChen.cons import (
    sql_write_path_hb,
    properties_com_k,
    properties_fin_k,
    db,
    db_tables,
    HSJY_EXCHANGE_INE,
    HSJY_EXCHANGE_CZCE,
    HSJY_EXCHANGE_SHFE,
    HSJY_EXCHANGE_DCE,
    HSJY_EXCHANGE_CFFEX,
    wind_sec
)
from hbshare.quant.CChen.load_data import load_calendar_extra
import hbshare as hbs
import pymysql
import re

pymysql.install_as_MySQLdb()

page_size_con = 49999


def trade_calendar(start_date=None, end_date=None, page_size=None):
    if start_date is None:
        start_date = datetime(2010, 1, 1).date()
    if end_date is None:
        end_date = datetime.now().date()
    if page_size is None:
        page_size = page_size_con

    sql = 'select distinct ENDDATE from ' + db_tables['hsjy_com_daily_quote'] \
          + ' where ENDDATE<=TRUNC(to_date(' + end_date.strftime('%Y%m%d') \
          + ', \'yyyymmdd\')) and ENDDATE>=TRUNC(to_date(' + start_date.strftime('%Y%m%d') \
          + ', \'yyyymmdd\')) order by ENDDATE'
    data = hbs.db_data_query(db=db, sql=sql, page_size=page_size)
    if data['pages'] > 1:
        for p in range(2, data['pages'] + 1):
            data['data'] = data['data'] + hbs.db_data_query(db=db, sql=sql, page_size=page_size, page_num=p)['data']

    return pd.to_datetime(pd.DataFrame(data['data'])['ENDDATE']).dt.date.tolist()


def wind_product_index_perc(
        start_date, end_date,
        db_path=sql_write_path_hb['daily'],
        table='fut_index_wind',
        min_volume=10000
):
    engine = create_engine(db_path)
    data = pd.read_sql_query(
        'select `code`, `name`, `t_date`, `close`, `product`, `volume` from ' + table
        + ' where `t_date`<=' + end_date.strftime('%Y%m%d')
        + ' and `t_date`>=' + start_date.strftime('%Y%m%d')
        + ' and `code` like \'%.WI\''
        + ' and `product` not in (\'index\') order by `t_date` desc',
        engine
    )

    products = data.drop_duplicates(['product'])['product'].reset_index(drop=True)

    price_perc = []
    name_list = []
    for i in range(len(products)):
        product_data = data[data['product'] == products[i]].reset_index(drop=True)
        if product_data['volume'][:60].mean() < min_volume:
            continue

        product_price_perc = (
                                product_data['close'][0] - min(product_data['close'])
                             ) / (
                                max(product_data['close']) - min(product_data['close'])
                            )
        price_perc.append(product_price_perc)
        name_list.append(product_data['name'][0].replace('指数', ''))

    result = pd.DataFrame(
        {
            'name': name_list,
            'price_perc': price_perc
        }
    )

    return result.sort_values(by='price_perc', ascending=False).reset_index(drop=True)


def wind_product_index_sigma_xs(
        start_date, end_date,
        db_path=sql_write_path_hb['daily'], table='fut_index_wind', freq=''
):
    engine = create_engine(db_path)
    data_raw = pd.read_sql_query(
        'select `code`, `name`, `t_date`, `close`, `product`, `volume` from ' + table
        + ' where `t_date`<=' + end_date.strftime('%Y%m%d')
        + ' and `t_date`>=' + start_date.strftime('%Y%m%d')
        + ' and `code` like \'%.WI\''
        + ' and `product` not in (\'index\') order by `t_date`',
        engine
    )

    calendar = load_calendar_extra(freq=freq, start_date=start_date, end_date=end_date, db_path=db_path)
    result = calendar.copy()

    products = data_raw.drop_duplicates(['product'])['product'].tolist()
    names = []
    for i in products:
        print(i)
        data = data_raw[data_raw['product'] == i].reset_index(drop=True)
        data_clean = calendar.merge(data[['t_date', 'close']], on='t_date')
        data_clean['ret'] = data_clean['close'] / data_clean['close'].shift(1) - 1
        result = result.merge(
            data_clean[['t_date', 'ret']].rename(
                columns={'ret': data['name'][0].replace('指数', '').replace('南华', '')}
            ),
            on='t_date', how='left'
        )
        names.append(data['name'][0].replace('指数', '').replace('南华', ''))
    result['sigma'] = result[names].apply(lambda x: np.std(x, ddof=1), axis=1)
    result['mean'] = result[names].apply(lambda x: np.nanmean(x), axis=1)
    return result


def wind_com_fut_sec_index(
        db_path,
        start_date=datetime(2015, 1, 1).date(),
        end_date=datetime.now().date(),
        table='fut_index_wind',
        decimal=100000000,
        key='amount',
):
    engine = create_engine(db_path)
    data = pd.read_sql_query(
        'select `code`, `name`, `t_date`, `close`, `product`, `volume`, `amount`, `oi_amount` from ' + table
        + ' where `t_date`<=' + end_date.strftime('%Y%m%d')
        + ' and `t_date`>=' + start_date.strftime('%Y%m%d')
        + ' and `code` like \'%.WI\''
        + ' and `product` in (\'index\') and `code`!=\'CCFI.WI\' order by `t_date`',
        engine
    )
    data['code'] = data['code'].apply(lambda x: wind_sec[x])
    df = data.pivot(index='t_date', columns='code', values=key)
    df /= decimal
    df.loc[df.index < datetime(2020, 1, 1).date(), :] = df.loc[df.index < datetime(2020, 1, 1).date(), :] / 2
    return df.reset_index()


def wind_fin_fut_index(
        db_path,
        start_date=datetime(2015, 1, 1).date(),
        end_date=datetime.now().date(),
        table='futures_wind',
        decimal=100000000,
        key='amount',
):
    engine = create_engine(db_path)
    data = pd.read_sql_query(
        'select `code`, `symbol`, `t_date`, `close`, `product`, `volume`, `amount`, `oi_amount` from ' + table
        + ' where `t_date`<=' + end_date.strftime('%Y%m%d')
        + ' and `t_date`>=' + start_date.strftime('%Y%m%d')
        + ' and `product` in (\'IC\', \'IF\', \'IH\', \'T\', \'TF\', \'TS\') order by `t_date`, `product`',
        engine
    )
    data = data[['t_date', 'product', key]].groupby(by=['t_date', 'product']).sum().round(2).reset_index()
    df = data.pivot(index='t_date', columns='product', values=key)
    df /= decimal
    return df.reset_index()


def fin_fut_daily_k_by_contracts(contract_list, start_date=None, end_date=None, page_size=None):
    if start_date is None:
        start_date = datetime(2010, 1, 1).date()
    if end_date is None:
        end_date = datetime.now().date()
    if page_size is None:
        page_size = page_size_con

    if len(contract_list) == 0:
        raise ValueError('code_list 未传入参数')
    elif len(contract_list) > 1:
        sql_contract = 'ContractInnerCode in ' + str(tuple(contract_list))
    elif len(contract_list) > 999:
        raise ValueError('code_list 参数过多')
    else:
        sql_contract = 'ContractInnerCode=' + str(contract_list[0])

    sql = 'select ' + ', '.join(properties_fin_k) + ' from ' + db_tables['hsjy_fin_daily_quote'] \
          + ' where ' + sql_contract + ' and TradingDay<=TRUNC(to_date(' + end_date.strftime('%Y%m%d') \
          + ', \'yyyymmdd\')) and TradingDay>=TRUNC(to_date(' + start_date.strftime('%Y%m%d') \
          + ', \'yyyymmdd\')) order by TradingDay, ContractInnerCode'

    data = hbs.db_data_query(db=db, sql=sql, page_size=page_size)
    if data['pages'] > 1:
        for p in range(2, data['pages'] + 1):
            data['data'] = data['data'] + hbs.db_data_query(db=db, sql=sql, page_size=page_size, page_num=p)['data']

    return data


def daily_ret_by_vol(start_date=None, end_date=None, ex_list=None, min_vol=10000, page_size=None):
    if ex_list is None:
        ex_list = [
            HSJY_EXCHANGE_SHFE,
            HSJY_EXCHANGE_DCE,
            HSJY_EXCHANGE_CZCE,
            HSJY_EXCHANGE_INE
        ]
    if start_date is None:
        start_date = datetime(2010, 1, 1).date()
    if end_date is None:
        end_date = datetime.now().date()
    if page_size is None:
        page_size = page_size_con

    if len(ex_list) == 0:
        raise ValueError('code_list 未传入参数')
    elif len(ex_list) > 1:
        sql_contract = 'EXCHANGE in ' + str(tuple(ex_list))
    elif len(ex_list) > 999:
        raise ValueError('code_list 参数过多')
    else:
        sql_contract = 'EXCHANGE=' + str(ex_list[0])

    sql = 'select ' + ', '.join(properties_com_k) + ' from ' + db_tables['hsjy_com_daily_quote'] \
          + ' where ' + sql_contract + ' and ENDDATE<=TRUNC(to_date(' + end_date.strftime('%Y%m%d') \
          + ', \'yyyymmdd\')) and ENDDATE>=TRUNC(to_date(' + start_date.strftime('%Y%m%d') \
          + ', \'yyyymmdd\')) and VOLUME>=' + str(min_vol) + ' order by ENDDATE, OPTIONCODE'
    data = hbs.db_data_query(db=db, sql=sql, page_size=page_size)
    if data['pages'] > 1:
        for p in range(2, data['pages'] + 1):
            data['data'] = data['data'] + hbs.db_data_query(db=db, sql=sql, page_size=page_size, page_num=p)['data']

    return data


def fut_com_amount_by_ex(start_date=None, end_date=None, page_size=None):

    ex_list = [
        HSJY_EXCHANGE_SHFE,
        HSJY_EXCHANGE_DCE,
        HSJY_EXCHANGE_CZCE,
        HSJY_EXCHANGE_INE
    ]
    if start_date is None:
        start_date = datetime(2010, 1, 1).date()
    if end_date is None:
        end_date = datetime.now().date()
    if page_size is None:
        page_size = page_size_con

    sql_e = 'EXCHANGE in ' + str(tuple(ex_list))

    sql = 'select ENDDATE, SUM(TURNOVER) from ' + db_tables['hsjy_com_daily_quote'] \
          + ' where ENDDATE<=TRUNC(to_date(' + end_date.strftime('%Y%m%d') \
          + ', \'yyyymmdd\')) and ENDDATE>=TRUNC(to_date(' + start_date.strftime('%Y%m%d') \
          + ', \'yyyymmdd\')) and ' + sql_e + ' group by ENDDATE order by ENDDATE'
    data = hbs.db_data_query(db=db, sql=sql, page_size=page_size)
    if data['pages'] > 1:
        for p in range(2, data['pages'] + 1):
            data['data'] = data['data'] + hbs.db_data_query(db=db, sql=sql, page_size=page_size, page_num=p)['data']

    return data


def rolling_volatility(data, vol_params=None):
    if vol_params is None:
        vol_params = [
            '20日', '30日', '60日', '90日', '120日', '150日', '200日'
        ]

    vol_max = []
    vol_75 = []
    vol_50 = []
    vol_25 = []
    vol_min = []
    vol_current = []

    for i in vol_params:
        data[i] = data['ret'].rolling(int(re.findall(r'\d+', i)[0])).std(ddof=1) * np.sqrt(250) * 100
        vol_max.append(np.nanmax(data[str(i)]))
        vol_75.append(np.nanpercentile(data[str(i)], 75))
        vol_50.append(np.nanpercentile(data[str(i)], 50))
        vol_25.append(np.nanpercentile(data[str(i)], 25))
        vol_min.append(np.nanmin(data[str(i)]))
        vol_current.append(data[str(i)].tolist()[-1])

    vol_data = pd.DataFrame(
        {
            '时间': vol_params,
            '最大值': vol_max,
            '75分位': vol_75,
            '中位值': vol_50,
            '25分位': vol_25,
            '最小值': vol_min,
            '最新值': vol_current
        }
    )
    return vol_data, data





"""
股票市场风格因子动量
"""
import os
import pandas as pd
import hbshare as hbs
from datetime import datetime, timedelta
from hbshare.quant.Kevin.rm_associated.config import style_names
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB

path = r'D:\kevin\risk_model_jy\RiskModel\data'


class StyleFactorMomentum:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_stock_style_momentum'
        self._load_calendar()

    def _load_calendar(self):
        pre_dt = datetime.strptime(self.start_date, '%Y-%m-%d') - timedelta(days=400)
        pre_date = pre_dt.strftime('%Y%m%d')
        end_date = self.end_date.replace('-', '')
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            pre_date, end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)

        self.calendar_df = df

    def get_factor_momentum(self):
        month_end_list = self.calendar_df[self.calendar_df['isMonthEnd'] == 1]['calendarDate'].tolist()
        trading_day_list = self.calendar_df[self.calendar_df['isOpen'] == 1]['calendarDate'].tolist()
        rank_ic_list = []
        for i in range(1, len(month_end_list)):
            m_end = month_end_list[i]
            pre_end = month_end_list[i - 1]
            period_date = [x for x in trading_day_list if pre_end < x <= m_end]
            # 本地取风格因子数据
            data_path = os.path.join(path, r'zzqz_sw\style_factor')
            style_factor = pd.read_csv(
                os.path.join(data_path, '{0}.csv'.format(pre_end)), dtype={"ticker": str}).set_index(
                'ticker')[style_names]
            # 本地行情数据
            data_path = os.path.join(path, r'common_data\chg_pct')
            period_info = []
            for date in period_date:
                chg_pct = pd.read_csv(
                    os.path.join(data_path, '{}.csv'.format(date)), dtype={"tradeDate": str, "ticker": str})
                period_info.append(chg_pct)

            period_info = pd.concat(period_info)
            pivot_df = pd.pivot_table(
                period_info, index='tradeDate', columns='ticker', values='dailyReturnReinv').fillna(0.).sort_index()
            period_ret = (1 + pivot_df).prod() - 1

            rank_ic = style_factor.apply(lambda x: x.corr(period_ret, method='spearman'), axis=0).to_frame(m_end).T
            rank_ic_list.append(rank_ic)

        ic_df = pd.concat(rank_ic_list)
        res = ic_df.rolling(12).mean().dropna().round(3)
        res.index.name = 'trade_date'
        res.reset_index(inplace=True)

        return res

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_factor_momentum()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_stock_style_momentum(
                id int auto_increment primary key,
                trade_date date not null unique,
                size decimal(4, 3),
                beta decimal(4, 3),
                momentum decimal(4, 3),
                earnyield decimal(4, 3),
                resvol decimal(4, 3),
                growth decimal(4, 3),
                btop decimal(4, 3),
                leverage decimal(4, 3),
                liquidity decimal(4, 3),
                sizenl decimal(4, 3)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_factor_momentum()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    # StyleFactorMomentum('2021-04-17', '2021-05-17', is_increment=1).get_construct_result()
    StyleFactorMomentum('2010-01-01', '2021-05-17', is_increment=0).get_construct_result()


# -*- coding: utf-8 -*-

from sqlalchemy import create_engine
from sqlalchemy.ext.compiler import compiles
from sqlalchemy.sql.expression import Insert
import pandas as pd
import pymysql
import sqlalchemy
import sys
import time
import traceback
import warnings
warnings.filterwarnings('ignore', category=pymysql.Warning)


class FEDB:
    def __init__(self):
        self.fedb_engine = None
        self.fedb_connection = None
        self.create_connection()

    def create_connection(self):
        vaild = False
        while not vaild:
            try:
                self.fedb_engine = create_engine('mysql+pymysql://{0}:{1}@{2}:{3}/{4}'.format(
                                                 'admin', 'mysql', '192.168.223.152', '3306', 'fe_temp_data'),
                                   connect_args={'charset': 'utf8'}, pool_recycle=360,
                                   pool_size=2, max_overflow=10, pool_timeout=360)
                self.fedb_connection = self.fedb_engine.connect()
                vaild = True
            except Exception as e:
                self.close_connection()
                print('[FEDB] mysql connect error')
                time.sleep(5)
        return

    def close_connection(self):
        # 1) close connection
        try:
            if self.fedb_connection is not None:
                self.fedb_connection.close()
        except Exception as e:
            print('[FEDB] close connect error')
        # 2) dispose engine
        try:
            if self.fedb_engine is not None:
                self.fedb_engine.dispose()
        except Exception as e:
            print('[FEDB] dispose engine error')
        return

    def get_df(self, sql):
        valid = False
        df = None
        while not valid:
            try:
                df = pd.read_sql_query(sql, self.fedb_engine)
                self.close_connection()
                valid = True
            except Exception as e:
                self.close_connection()
                self.create_connection()
                print('[FEDB] sql error:{0}'.format(sql))
                time.sleep(5)
        return df

    def create_metadata(self, metadata):
        valid = False
        while not valid:
            try:
                metadata.create_all(self.fedb_engine)
                valid = True
            except Exception as e:
                self.close_connection()
                self.create_connection()
                exc_type, exc_value, exc_trackback = sys.exc_info()
                print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
                print('[FEDB] create metadata error')
            return metadata

    def insert_data(self, table, params):
        try:
            self.fedb_connection.execute(table.insert(), params)
        except Exception as e:
            conn = self.fedb_engine.connect()
            conn.execute(table.insert(), params)
            self.fedb_connection = conn
            exc_type, exc_value, exc_trackback = sys.exc_info()
            print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
            print('[FEDB] insert data error')
        return

    def replace_data(self, table, params):
        # @compiles(Insert)
        def replace_string(insert, compiler, **kw):
            s = compiler.visit_insert(insert, **kw)
            s = s.replace("INSERT INTO", "REPLACE INTO")
            return s

        valid = False
        times = 0
        while not valid and times < 5:
            try:
                times += 1
                self.fedb_connection.execute(table.insert(replace_string=""), params)
                valid = True
            except Exception as e:
                self.close_connection()
                self.create_connection()
                time.sleep(2)
                exc_type, exc_value, exc_trackback = sys.exc_info()
                print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
                print('[FEDB] replace data error')
            return

    def update_data(self, table, params):
        # @compiles(Insert)
        def append_string(insert, compiler, **kw):
            s = compiler.visit_insert(insert, **kw)
            if insert.kwards.get('on_duplicates_key_update'):
                fields = s[s.find("(") + 1:s.find(")")].replace(" ", "").split(",")
                generated_directive = ["{0}=VALUES({0})".format(field) for field in fields]
                return s + " ON DUPLICATES KEY UPDATE " + ",".join(generated_directive)
            return s

        valid = False
        times = 0
        while not valid and times < 5:
            try:
                times += 1
                self.fedb_connection.execute(table.insert(on_duplicates_key_update=True), params)
                valid = True
            except Exception as e:
                self.close_connection()
                self.create_connection()
                time.sleep(2)
                exc_type, exc_value, exc_trackback = sys.exc_info()
                print(repr(traceback.format_exception(exc_type, exc_value, exc_trackback)))
                print('[FEDB] update data error')
            return

    def insert_df(self, factor_df):
        table_name = 'mutual_fund_holding_label'
        metadata = sqlalchemy.MetaData()
        table = sqlalchemy.Table(
            table_name, metadata,
            sqlalchemy.Column('REPORT_DATE'),
            sqlalchemy.Column('REPORT_HISTORY_DATE'),
            sqlalchemy.Column('FUND_UNIVERSE'),
            sqlalchemy.Column('IS_ZC'),
            sqlalchemy.Column('LABEL_TYPE'),
            sqlalchemy.Column('LABEL_NAME'),
            sqlalchemy.Column('LABEL_VALUE'),
            sqlalchemy.Column('LABEL_VALUE_STRING')
        )
        metadata = self.create_metadata(metadata)
        params = []
        for idx in range(len(factor_df)):
            rows = factor_df.iloc[idx]
            param = dict()
            param['REPORT_DATE'] = rows['REPORT_DATE']
            param['REPORT_HISTORY_DATE'] = rows['REPORT_HISTORY_DATE']
            param['FUND_UNIVERSE'] = rows['FUND_UNIVERSE']
            param['IS_ZC'] = rows['IS_ZC']
            param['LABEL_TYPE'] = rows['LABEL_TYPE']
            param['LABEL_NAME'] = rows['LABEL_NAME']
            param['LABEL_VALUE'] = rows['LABEL_VALUE']
            param['LABEL_VALUE_STRING'] = rows['LABEL_VALUE_STRING']
            params.append(param)
            if len(params) > 1000:
                self.update_data(table, params)
                del params
                params = []
        if len(params) > 0:
            self.update_data(table, params)
        self.close_connection()
        return

    def read_data(self, date, label_type):
        sql = "SELECT REPORT_DATE, REPORT_HISTORY_DATE, FUND_UNIVERSE, IS_ZC, LABEL_TYPE, LABEL_NAME, LABEL_VALUE, LABEL_VALUE_STRING FROM fe_temp_data.mutual_fund_holding_label where REPORT_DATE='{0}' AND LABEL_TYPE='{1}' AND FUND_UNIVERSE='FOCUS_MUTUAL_FUND_UNIVERSE' AND IS_ZC=1".format(date, label_type)
        df = self.get_df(sql)
        df['REPORT_DATE'] = df['REPORT_DATE'].astype(str)
        df['REPORT_HISTORY_DATE'] = df['REPORT_HISTORY_DATE'].astype(str)
        df['IS_ZC'] = df['IS_ZC'].astype(int)
        return df


"""
经济周期判断程序
"""
import pandas as pd
from datetime import datetime
# from hbshare.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class EconomicCycle:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_cycle'

    def get_eco_increase_data(self):
        """
        经济增长类数据
        """
        index_dict = {
            "M5525763": "社融增速",
            "M0001385": "M2增速",
            "M0039354": "GDP增速",
            "M0017126": "制造业PMI",
            "S0027013": "发电量同比",
            "M0012303": "消费者信心指数",
            "M0001428": "社零总额增速",
            "M1001654": "十年期国债到期收益率",
            "M1001646": "一年期国债到期收益率",
            "S5042881": "商品价格指数",
            "M0000545": "工业增加值",
            "M5650805": "失业率",
            "M5792266": "产能利用率",
            "S2707396": "30大中城市_商品房成交面积_当月值",
            "M0000273": "固定资产投资累计同比",
            "M0000609": "进口金额同比",
            "M0000557": "工业企业利润",
            "S0206721": "制造业利润总额累计同比",
            "S0206720": "采矿业利润总额累计同比",
            "S0206722": "电热燃气及水的生产和供应业利润总额累计同比",
            "S6114596": "汽车销量当月同比",
            "S0028199": "彩电产量当月同比"
                      }
        res = w.edb(','.join(list(index_dict.keys())), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch economic cycle data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=index_dict, inplace=True)

        date_list = [datetime.strftime(x, '%Y%m%d') for x in pd.date_range(self.start_date, self.end_date, freq='M')]
        data = data.set_index('trade_date').fillna(method='ffill').reindex(date_list)
        data['社融与M2增速差'] = data['社融增速'] - data['M2增速']
        data['国债期限利差'] = data['十年期国债到期收益率'] - data['一年期国债到期收益率']
        data['30大中城市商品房成交面积同比'] = data['30大中城市_商品房成交面积_当月值'].pct_change(periods=12)
        data = data.drop(
            ['社融增速', 'M2增速', '十年期国债到期收益率', '一年期国债到期收益率', '30大中城市_商品房成交面积_当月值'], axis=1)
        # 负向调整
        data['失业率'] *= -1

        return data

    def get_liquidity_data(self):
        """
        宏观流动性数据
        """
        index_dict = {
            "S2707412": "70个大中城市新建商品住宅价格指数环比",
            "M5525763": "社融增速",
            "M0096870": "贷款市场报价利率LPR_一年",
            "M0010055": "银行间同业拆借利率加权平均值_14天",
            "M0017140": "SHIBOR_2W",
            "M6404532": "实体经济杠杆率",
            "M0009970": "信贷余额",
            "M0017126": "制造业PMI",
            "M1008942": "信用利差",
            "M0001383": "M1同比",
            "M0001385": "M2同比",
            "M0041371": "七天逆回购利率",
            "M0009937": "银行间质押式回购加权利率",
            "M0043821": "中小机构存款准备金率",
            "M0061518": "大型机构存款准备金率",
            "M1004907": "国开债到期收益率_1Y",
            "M1004906": "国开债到期收益率_6M",
            "M0010096": "超储率"
        }

        res = w.edb(','.join(list(index_dict.keys())), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch economic cycle data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=index_dict, inplace=True)

        date_list = [datetime.strftime(x, '%Y%m%d') for x in pd.date_range(self.start_date, self.end_date, freq='M')]
        data = data.set_index('trade_date').fillna(method='ffill').reindex(date_list)

        data['国开债到期收益率期现差值'] = data['国开债到期收益率_1Y'] - data['国开债到期收益率_6M']
        data = data.drop(['国开债到期收益率_1Y', '国开债到期收益率_6M'], axis=1)
        # 负向调整
        neg_cols = ['贷款市场报价利率LPR_一年', '银行间同业拆借利率加权平均值_14天', 'SHIBOR_2W', '七天逆回购利率',
                    '银行间质押式回购加权利率', '中小机构存款准备金率', '大型机构存款准备金率', '国开债到期收益率期现差值', '超储率']
        for col in neg_cols:
            data[col] *= -1

        return data

    def calculate_economic_cycle(self):
        eco_increase_data = self.get_eco_increase_data()
        currency_data = self.get_liquidity_data()

        increase_diff = eco_increase_data.diff().dropna(how='all')
        increase_diff['制造业PMI'] = eco_increase_data['制造业PMI'][1:]
        increase_diff['制造业PMI'] -= 50.
        increase_count_df = increase_diff.gt(0.).sum(axis=1).to_frame('增长观点个数').merge(
            increase_diff.eq(0.).sum(axis=1).to_frame('维持不变个数'), left_index=True, right_index=True).merge(
            increase_diff.lt(0.).sum(axis=1).to_frame('衰退观点个数'), left_index=True, right_index=True)
        increase_count_df['经济增长概率'] = increase_count_df['增长观点个数'] / (increase_count_df.sum(axis=1))

        currency_diff = currency_data.diff().dropna(how='all')
        currency_diff['制造业PMI'] = currency_data['制造业PMI'][1:]
        currency_diff['制造业PMI'] -= 50.
        currency_count_df = currency_diff.gt(0.).sum(axis=1).to_frame('增长观点个数').merge(
            currency_diff.eq(0.).sum(axis=1).to_frame(name='维持不变个数'), left_index=True, right_index=True).merge(
            currency_diff.lt(0.).sum(axis=1).to_frame(name='衰退观点个数'), left_index=True, right_index=True)
        currency_count_df['宽松概率'] = currency_count_df['增长观点个数'] / (currency_count_df.sum(axis=1))

        return increase_count_df, currency_count_df


if __name__ == '__main__':
    EconomicCycle('2008-12-20', '2021-06-08', is_increment=0).calculate_economic_cycle()



# -*- coding:utf-8 -*- 
import sys
PY3 = (sys.version_info[0] >= 3)
HTTP_OK = 200
HTTP_AUTHORIZATION_ERROR = 401
HTTP_URL = 'data.howbuy.com'
HTTP_PORT = 443
TOKEN_F_P = 'tk.csv'
TOKEN_ERR_MSG = '请设置hbshare的token凭证码，如果没有请联系好买FDC注册申请'

ENV_F_P = 'env.csv'
ENV_ERR_MSG = '调用接口涉及环境参数，请设置hbshare的环境参数'
ENV_PROD = 'prod'




from setuptools import setup, find_packages
import codecs
import os
import sys


def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()

def write(fname, ver):
    file = codecs.open(os.path.join(os.path.dirname(__file__), fname),'w')
    file.write(ver)
    file.close()

def get_version_code(args):
    print(args)
    version_path = 'hbshare/VERSION.txt'
    ver_code = read(version_path)
    if len(args) >= 2 and args[1] == 'sdist':
        vers = ver_code.replace('.','')
        l_ver = int(vers) + 1
        ver_code = ('%s.%s.%s') % (l_ver // 100, ((l_ver // 10) % 10), (l_ver % 10))
        write(version_path, ver_code)
    return ver_code


long_desc = """
HBShare
===============
Target Users
--------------

* financial market analyst of China
* learners of financial data analysis with pandas/NumPy
* people who are interested in China financial data

Installation
--------------

    pip install hbshare
    
Upgrade
---------------

    pip install hbshare --upgrade
    
Quick Start
--------------

::

    import hbshare as hbs
    
    hbs.set_token("XXXX")
    data = hbs.get_fund_newest_nav_by_code('000004')
    
return::

#     jjdm      jzrq   jjjz   ljjz    hbdr     hb1y     hb3y     hb6y     hbjn      hb1n  zfxz
0  000004  20200612  0.758  0.968  0.1321 -3.92902 -9.11271  1.74497 -7.22154  11.63476   3.0

"""

def read_install_requires():

    reqs = [
            'pandas>=0.18.0',
            'requests>=2.0.0',
            'simplejson>=3.16.0'
            ]
    return reqs

def read_file(file):
    with open(file, "rt") as f:
        return f.read()

#get_new_version_code()

setup(
    name='hbshare',
    version=read('hbshare/VERSION.txt'),
    #version='1.7.1',
    description='Howbuy Quantitative Research SDK.'
                'Fund product data query and withdrawal tools provided by Howbuy fund company for investment research',
    # long_description=read("READM.rst"),
    # long_description = long_desc,
    author='meng.lv',
    author_email='49007952@qq.com',
    maintainer='meng.lv',
    maintainer_email='49007952@qq.com',
    license='BSD',
    url='https://www.howbuy.com',
    install_requires=read_install_requires(),
    keywords='Global Financial Data',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: BSD License'],
    packages=find_packages(),
    include_package_data=True,
    package_dir={'hbshare':'hbshare'},
    package_data={'hbshare': ['*.csv', '*.txt']},
)


import numpy as np

# xlsx params
row_num = 3
module_2_cols = 12


# xlwings format
cell_format = {
    'type': '3_color_scale',
    'min_color': '#2EDFA3',
    'mid_color': '#F3F8F1',
    'max_color': '#f47983'
}

cell_format_r = {
    'type': '3_color_scale',
    'min_color': '#f47983',
    'mid_color': '#F3F8F1',
    'max_color': '#2EDFA3'
}

cell_format2 = {
    'type': 'data_bar',
    'bar_color': 'red',
    'bar_only': False,
    'bar_solid': False,
    'bar_negative_color': 'green',
    'bar_direction': 'left',
    'bar_axis_position': 'middle'
}


# 10进制转26进制
sequence = list(map(lambda x: chr(x), range(ord('A'), ord('Z') + 1)))


def ten2twentysix(num):
    lll = []
    if num > 25:
        while 1:
            d = int(num / 26)
            remainder = num % 26
            if d <= 25:
                lll.insert(0, sequence[remainder])
                lll.insert(0, sequence[d - 1])
                break
            else:
                lll.insert(0, sequence[remainder])
                num = d - 1
    else:
        lll.append(sequence[num])

    return ''.join(lll)


# 基金表现统计
def pool_board(data_df, fund_list):
    ret_this_week = data_df[
        data_df['index'] == '本周'
        ].drop(
        columns={'index'}
    ).reset_index(drop=True).T.rename(
        columns={0: 'return'}
    )
    ret_this_week = ret_this_week[
        np.isnan(ret_this_week['return'].tolist()) == False
        ].sort_values(by='return', ascending=False).reset_index()

    result = {
        'all': data_df.drop(columns={'index'}).shape[1],
        'onTime': len(ret_this_week),
        'win': len(ret_this_week[ret_this_week['return'] > 0]),
        'bestName': ret_this_week['index'][0],
        'bestRet': ret_this_week['return'][0],
        'bestType': fund_list[fund_list['name'] == ret_this_week['index'][0]]['type'].tolist()[0],
        'bestName1': ret_this_week['index'][1],
        'bestRet1': ret_this_week['return'][1],
        'bestType1': fund_list[fund_list['name'] == ret_this_week['index'][1]]['type'].tolist()[0],
        'bestName2': ret_this_week['index'][2],
        'bestRet2': ret_this_week['return'][2],
        'bestType2': fund_list[fund_list['name'] == ret_this_week['index'][2]]['type'].tolist()[0],
        'worstName': ret_this_week['index'][len(ret_this_week) - 1],
        'worstRet': ret_this_week['return'][len(ret_this_week) - 1],
        'worstType': fund_list[
            fund_list['name'] == ret_this_week['index'][len(ret_this_week) - 1]
            ]['type'].tolist()[0],
        'worstName1': ret_this_week['index'][len(ret_this_week) - 2],
        'worstRet1': ret_this_week['return'][len(ret_this_week) - 2],
        'worstType1': fund_list[
            fund_list['name'] == ret_this_week['index'][len(ret_this_week) - 2]
            ]['type'].tolist()[0],
        'worstName2': ret_this_week['index'][len(ret_this_week) - 3],
        'worstRet2': ret_this_week['return'][len(ret_this_week) - 3],
        'worstType2': fund_list[
            fund_list['name'] == ret_this_week['index'][len(ret_this_week) - 3]
            ]['type'].tolist()[0],
        'average': ret_this_week['return'].mean()
    }
    return result


if __name__ == '__main__':
    print(ten2twentysix(0))



import pandas as pd
import simdjson

from FundManager.database import *


if __name__ == "__main__":
	# get_manager_basic_info(
	# 	['RYXB', 'RYXM', 'RYXL', 'CYNX', 'RZRQ', 'LRRQ', 'RYZT', 'JJDM', 'GGRQ', "RYDM"], None
	# ).sort_values(["人员代码", "入职日期"]).to_csv("基金经理信息.csv", index=False)
	all_days = len(get_all_trading_day('20210101'))
	fund_net_worth_days = pd.DataFrame(get_fund_net_worth_days('20210101'))
	weekly_funds = []
	for index, row in fund_net_worth_days.iterrows():
		if row.COUNT < all_days:
			weekly_funds.append({'code': row.JJDM, 'count': row.COUNT, 'max_date': row.MAX})
	print(len(weekly_funds) / all_days)
	# get_public_fund_net_value(
	# 	["JJDM", "JZRQ", "LJJZ", "JJJZ"], None
	# ).sort_values(["基金代码", "净值日期"]).to_csv("基金净值信息.csv", index=False)
	# simdjson.dump(get_all_fund_category(), open('database/公募基金一级分类.json', 'w'), indent=2, ensure_ascii=False)
	# simdjson.dump(get_all_fund_category(is_first=False), open('database/公募基金二级分类.json', 'w'), indent=2, ensure_ascii=False)





"""
晨星基金风格箱模型
"""
import pandas as pd
import numpy as np
import statsmodels.api as sm
from hbshare.fe.common.util.data_loader import MorningStarBoxStyleLoader
from hbshare.fe.common.util.verifier import verify_type, verify_simple_data_duplicate


class MorningStarStyleCalculator:
    def __init__(self, trade_date, mode="main"):
        """
        @param trade_date: 月末的交易日期
        @param mode: 模式，"main"代表是十大重仓的风格，"all"代表全持仓风格
        """
        self.trade_date = trade_date
        self.mode = mode
        self._load_data()
        self._verify()
        self._init_data_param()

    def _load_data(self):
        self.data_param = MorningStarBoxStyleLoader(self.trade_date, self.mode).load()

    def _verify(self):
        if not self.data_param:
            return
        equity_value = self.data_param.get('equity_value')
        verify_type(equity_value, 'equity_value', pd.DataFrame)
        verify_simple_data_duplicate(equity_value.index, 'equity_value.index')

        equity_growth = self.data_param.get('equity_growth')
        verify_type(equity_growth, 'equity_growth', pd.DataFrame)

        fund_holding = self.data_param.get('fund_holding')
        verify_type(fund_holding, 'fund_holding', pd.DataFrame)

    def _init_data_param(self):
        if not self.data_param:
            return
        equity_value = self.data_param.get('equity_value')
        equity_growth = self.data_param.get('equity_growth')
        fund_holding = self.data_param.get('fund_holding')

        self.quotation_date = self.data_param.get('quotation_date')
        idx = equity_value.index.intersection(equity_growth['ticker'].unique())
        self.equity_value = equity_value.reindex(idx)
        self.equity_growth = equity_growth[equity_growth['ticker'].isin(idx)]
        fund_holding = pd.pivot_table(fund_holding, index='ticker', columns='fund_id', values='weight').fillna(0.)
        idx = idx.intersection(fund_holding.index)
        fund_holding = fund_holding.reindex(idx)
        cols = fund_holding.sum()[fund_holding.sum() > 0.].index.tolist()
        self.fund_holding = fund_holding[cols]

    @staticmethod
    def _cal_cap_bound(equity_df, lower=0.75, upper=0.90):
        df = equity_df.copy()
        df.sort_values(by='marketValue', ascending=False, inplace=True)
        df['tmp'] = (df['marketValue'] / df['marketValue'].sum()).cumsum()
        lmt = df[df['tmp'] <= lower]['marketValue'].min()
        mst = df[df['tmp'] <= upper]['marketValue'].min()
        del df['tmp']
        l_cap = df[df['marketValue'] >= lmt]
        m_cap = df[(df['marketValue'] < lmt) & (df['marketValue'] >= mst)]
        s_cap = df[df['marketValue'] < mst]

        return mst, lmt, l_cap, m_cap, s_cap

    @staticmethod
    def _cal_value_score(data):
        df = data.copy()
        df['EP'] = 1.0 / df['PE']
        df['BP'] = 1.0 / df['PB']
        df['CFP'] = 1.0 / df['PCF']
        factor_list = ['EP', 'BP', 'CFP', 'DIVIDENDRATIO']
        value_df = df[factor_list].dropna(how='all').fillna(0.)
        value_df = value_df.apply(lambda x: x.rank() / len(x.dropna()))

        weight = pd.DataFrame(
            [[1. / len(factor_list)] * len(factor_list)] * len(value_df), index=value_df.index, columns=factor_list)
        weight = ~pd.isnull(value_df) * weight
        weight = weight.divide(weight.sum(axis=1), axis=0)

        value_series = weight.multiply(value_df).sum(axis=1)

        return value_series

    @staticmethod
    def _cal_growth_score(data):
        df = data.copy()
        factor_list = ['revenue', 'profit', 'asset']
        growth_df = df[factor_list].dropna(how='all')
        growth_df = growth_df.apply(lambda x: x.rank() / len(x.dropna()))

        weight = pd.DataFrame(
            [[1. / len(factor_list)] * len(factor_list)] * len(growth_df), index=growth_df.index, columns=factor_list)
        weight = ~pd.isnull(growth_df) * weight
        weight = weight.divide(weight.sum(axis=1), axis=0)

        growth_series = weight.multiply(growth_df).sum(axis=1)

        return growth_series

    @staticmethod
    def _lg(df):
        flag = sm.add_constant(np.arange(len(df)))
        ols_fit = sm.OLS(df.values, flag).fit()
        rate = ols_fit.params[1] / df.abs().mean()

        return rate

    def _calculate_growth_score(self):
        data = self.equity_growth.copy()
        revenue_cagr = pd.pivot_table(
            data, index='endDate', columns='ticker', values='revenue').sort_index().apply(self._lg)
        profit_cagr = pd.pivot_table(
            data, index='endDate', columns='ticker', values='net_profit').sort_index().apply(self._lg)
        asset_cagr = pd.pivot_table(
            data, index='endDate', columns='ticker', values='t_asset').sort_index().apply(self._lg)
        growth_score = revenue_cagr.to_frame('revenue').merge(
            profit_cagr.to_frame('profit'), left_index=True, right_index=True).merge(
            asset_cagr.to_frame('asset'), left_index=True, right_index=True)

        return growth_score

    def get_equity_score(self, growth_score):
        equity_df = self.equity_value.merge(growth_score, left_index=True, right_index=True)
        mst, lmt, l_cap, m_cap, s_cap = self._cal_cap_bound(equity_df)
        cut_list = [l_cap, m_cap, s_cap]
        raw_x_list = []
        for df in cut_list:
            value_series = self._cal_value_score(df)
            growth_series = self._cal_growth_score(df)
            vcg_series = (growth_series - value_series).dropna()

            vcg_df = pd.merge(
                vcg_series.to_frame('vcg'), equity_df['negMarketValue'],
                left_index=True, right_index=True).sort_values(by='vcg')
            vcg_df['tmp'] = vcg_df['negMarketValue'].cumsum() / vcg_df['negMarketValue'].sum()
            gt = vcg_df[vcg_df['tmp'] <= 2. / 3]['vcg'].max()
            vt = vcg_df[vcg_df['tmp'] <= 1. / 3]['vcg'].max()

            raw_x_list.append(100 * (1 + (vcg_series - vt) / (gt - vt)))

        raw_x = pd.concat(raw_x_list)
        raw_y = 100 * (1 + (np.log(equity_df['marketValue']) - np.log(mst)) / (np.log(lmt) - np.log(mst)))
        raw_df = pd.merge(raw_x.to_frame('vcg_score'), raw_y.to_frame('cap_score'),
                          left_index=True, right_index=True).reset_index()
        raw_df.rename(columns={"index": "ticker"}, inplace=True)

        return raw_df

    def get_construct_results(self):
        growth_score = self._calculate_growth_score()
        equity_score = self.get_equity_score(growth_score)

        vcg_series = equity_score.set_index('ticker')['vcg_score'].reindex(self.fund_holding.index)
        cap_series = equity_score.set_index('ticker')['cap_score'].reindex(self.fund_holding.index)
        holding_df = self.fund_holding.div(self.fund_holding.sum(), axis=1)
        fund_vcg = holding_df.T.dot(vcg_series)
        fund_cap = holding_df.T.dot(cap_series)
        fund_score = pd.merge(fund_cap.to_frame('cap_score'), fund_vcg.to_frame('vcg_score'),
                              left_index=True, right_index=True).reset_index()
        # category
        fund_score.loc[fund_score['cap_score'] < 100, 'cap_cate'] = "小盘"
        fund_score.loc[(fund_score['cap_score'] >= 100) & (fund_score['cap_score'] <= 200), 'cap_cate'] = "中盘"
        fund_score.loc[fund_score['cap_score'] > 200, 'cap_cate'] = "大盘"
        fund_score.loc[fund_score['vcg_score'] < 125, 'vcg_cate'] = "价值型"
        fund_score.loc[(fund_score['vcg_score'] >= 125) & (fund_score['vcg_score'] <= 175), 'vcg_cate'] = "平衡型"
        fund_score.loc[fund_score['vcg_score'] > 175, 'vcg_cate'] = "成长型"
        fund_score['category'] = fund_score['cap_cate'] + fund_score['vcg_cate']
        fund_score['type'] = self.mode
        # fund_score['trade_date'] = self.trade_date
        fund_score['trade_date'] = self.data_param['report_date']
        fund_score = fund_score[['trade_date', 'fund_id', 'type', 'cap_score', 'vcg_score', 'category']]

        # equity style
        equity_score.loc[equity_score['cap_score'] < 100, 'cap_cate'] = "小盘"
        equity_score.loc[(equity_score['cap_score'] >= 100) & (equity_score['cap_score'] <= 200), 'cap_cate'] = "中盘"
        equity_score.loc[equity_score['cap_score'] > 200, 'cap_cate'] = "大盘"
        equity_score.loc[equity_score['vcg_score'] < 100, 'vcg_cate'] = "价值型"
        equity_score.loc[(equity_score['vcg_score'] >= 100) & (equity_score['vcg_score'] <= 200), 'vcg_cate'] = "均衡型"
        equity_score.loc[equity_score['vcg_score'] > 200, 'vcg_cate'] = "成长型"
        equity_score['category'] = equity_score['cap_cate'] + equity_score['vcg_cate']
        equity_score['type'] = self.mode
        # equity_score['trade_date'] = self.quotation_date
        equity_score['trade_date'] = self.data_param['report_date']
        equity_score = equity_score[['trade_date', 'ticker', 'type', 'cap_score', 'vcg_score', 'category']]

        # wind_fund_cate = pd.read_excel("D:\\kevin\\Wind_2020_annual_style_cate.xlsx").dropna()
        # wind_fund_cate.columns = ['fund_id', 'name', 'fund_type', 'category_wind']
        # wind_fund_cate['fund_id'] = wind_fund_cate['fund_id'].apply(lambda x: x.split('.')[0])
        # wind_fund_cate['category_wind'] = wind_fund_cate['category_wind'].apply(lambda x: x[:4] + '型')
        # df = pd.merge(fund_score[['fund_id', 'category']], wind_fund_cate, on='fund_id')
        # df['cap_cate'] = df['category'].apply(lambda x: x[:2])
        # df['cap_cate_wind'] = df['category_wind'].apply(lambda x: x[:2])
        # df['vcg_cate'] = df['category'].apply(lambda x: x[2:])
        # df['vcg_cate_wind'] = df['category_wind'].apply(lambda x: x[2:])

        return {"equity_score": equity_score, "fund_score": fund_score}


if __name__ == '__main__':
    # 跑历史
    # from hbshare.rm_associated.util.data_loader import get_trading_day_list
    # date_list = get_trading_day_list('20100101', '20211115', frequency='month')
    #
    # for date in date_list:
    #     if date[4:6] in ['01', '07', '10']:
    #         res = MorningStarStyleCalculator(date, mode='main').get_construct_results()
    #     elif date[4:6] in ['04', '08']:
    #         res = MorningStarStyleCalculator(date, mode='all').get_construct_results()
    #         print(res)
    #     else:
    #         res = None
    #     print("当期日期: {}".format(date))
    #     print(res)
    res = MorningStarStyleCalculator('20210430', mode='main').get_construct_results()
    print(res)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @FileName  :HighDataIndex.py
# @Time      :2021/4/25 3:52 下午
# @Author    :meng.lv

from hbshare.base import BaseIndex
from hbshare.db.simu import cons as ct
import os, sys
"""
 私募指标
"""

class SimuIndex(BaseIndex.BaseIndex):

    def __init__(self, key, show_log=False):
        path = os.path.abspath(os.path.dirname(__file__)) + "/simu.csv"
        super(SimuIndex, self).__init__("highuser", path)
        self.key = key
        self.show_log = show_log
        self.sql = super(SimuIndex, self).sql(self.key)
        self.print_msg(self.sql)

    def set_key(self, key):
        self.key = key
        self.sql = super(SimuIndex, self).sql(self.key)
        self.print_msg(self.sql)

    def print_msg(self, msg):
        if self.show_log:
            print("Output:",msg)

    def get(self, jjdm, time_dim_val, time_solt=None, benchmark=None):
        #检查时间格式
        time_dim_val = super(SimuIndex, self).check_dim_time(self.key, time_dim_val)
        l_sql = self.sql % (jjdm, time_dim_val)
        self.print_msg(l_sql)
        data = super(SimuIndex, self).call(l_sql)
        if time_solt != None:
            data = data.drop(index=data[~data['zblb'].isin(time_solt)].index.tolist())

        if benchmark != None:
            data = data.drop(index=data[~data['jzdm'].isin(benchmark)].index.tolist())
        return data

def simu_index(key, jjdm, dim_time, time_solt=None, benchmark=None, show_log=True):
    simuIdx = SimuIndex(key, show_log)
    return simuIdx.get(jjdm, dim_time, time_solt)




"""
私募基金深度报告业绩分析模块
"""
import pandas as pd
import numpy as np
from datetime import datetime
import hbshare as hbs
from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_fund_nav_from_sql, get_trading_day_list
from Arbitrage_backtest import cal_annual_return, cal_annual_volatility, cal_sharpe_ratio, cal_max_drawdown
import plotly
from plotly.offline import plot as plot_ly
import plotly.graph_objs as go
import plotly.figure_factory as ff

plotly.offline.init_notebook_mode(connected=True)


class FundNavAnalysor:
    def __init__(self, start_date, end_date, benchmark_id, nav_series):
        self.start_date = start_date
        self.end_date = end_date
        self.benchmark_id = benchmark_id
        self.nav_series = nav_series
        self._load_data()

    def _load_data(self):
        trading_day_list = get_trading_day_list(self.start_date, self.end_date, frequency="week")
        nav_series = self.nav_series.reindex(trading_day_list)
        # benchmark
        sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                     "and JYRQ >= {} and JYRQ <= {}".format(self.benchmark_id, self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script)
        map_dict = {"000905": "中证500", "000300": "沪深300", "000852": "中证1000"}
        benchmark_name = map_dict[self.benchmark_id]
        data = pd.DataFrame(res['data']).rename(columns={"TCLOSE": benchmark_name}).set_index(
            'TRADEDATE')[[benchmark_name]]
        benchmark_df = data.reindex(trading_day_list)

        assert (nav_series.shape[0] == benchmark_df.shape[0])

        nav_df = pd.merge(nav_series, benchmark_df, left_index=True, right_index=True)
        return_df = nav_df.pct_change().fillna(0.)
        return_df['超额'] = return_df[return_df.columns[0]] - return_df[benchmark_name]

        self.nav_df = (1 + return_df).cumprod()
        self.return_df = return_df[1:]

    @staticmethod
    def plotly_line(df, title_text, sava_path, figsize=(1200, 500)):
        fig_width, fig_height = figsize
        data = []

        for col in df.columns[:2]:
            trace = go.Scatter(
                x=df.index.tolist(),
                y=df[col],
                name=col,
                mode="lines"
            )
            data.append(trace)

        for col in df.columns[2:]:
            trace = go.Scatter(
                x=df.index.tolist(),
                y=df[col],
                name=col,
                mode="lines",
                line=dict(color='darkgray', width=2, dash='dot')
            )
            data.append(trace)

        date_list = df.index.tolist()
        n = int(len(date_list) / 12)
        tick_vals = [i for i in range(0, len(df), n)]
        tick_text = [date_list[i] for i in range(0, len(df), n)]

        layout = go.Layout(
            title=dict(text=title_text),
            autosize=False, width=fig_width, height=fig_height,
            yaxis=dict(tickfont=dict(size=12), showgrid=True),
            xaxis=dict(showgrid=True, tickvals=tick_vals, ticktext=tick_text),
            legend=dict(orientation="h", x=0.35, y=1.1),
            template='simple_white',
            # paper_bgcolor='#edeeee',
            # plot_bgcolor='#edeeee',
        )
        fig = go.Figure(data=data, layout=layout)

        plot_ly(fig, filename=sava_path, auto_open=False)

    def run(self):
        nav_df = self.nav_df.copy()
        return_df = self.return_df.copy()
        # 净值指标
        performance_df = pd.DataFrame(
            index=nav_df.columns, columns=["累计收益", "年化收益率", "年化波动率", "最大回撤",
                                           "Sharpe比率", "Calmar比率", "投资胜率", "平均损益比"])
        performance_df.loc[:, "累计收益"] = nav_df.iloc[-1] - 1
        performance_df.loc[:, "年化收益率"] = return_df.apply(cal_annual_return, axis=0)
        performance_df.loc[:, '年化波动率'] = return_df.apply(cal_annual_volatility, axis=0)
        performance_df.loc[:, "最大回撤"] = nav_df.apply(cal_max_drawdown, axis=0)
        performance_df.loc[:, "Sharpe比率"] = return_df.apply(lambda x: cal_sharpe_ratio(x, 0.015), axis=0)
        performance_df['Calmar比率'] = performance_df['年化收益率'] / performance_df['最大回撤'].abs()
        performance_df.loc[:, "投资胜率"] = return_df.apply(lambda x: x.gt(0).sum() / len(x), axis=0)
        performance_df.loc[:, "平均损益比"] = return_df.apply(lambda x: x[x > 0].mean() / x[x < 0].abs().mean(), axis=0)
        # 格式处理
        performance_df['累计收益'] = performance_df['累计收益'].apply(lambda x: format(x, '.2%'))
        performance_df['年化收益率'] = performance_df['年化收益率'].apply(lambda x: format(x, '.2%'))
        performance_df['年化波动率'] = performance_df['年化波动率'].apply(lambda x: format(x, '.2%'))
        performance_df['最大回撤'] = performance_df['最大回撤'].apply(lambda x: format(x, '.2%'))
        performance_df['Sharpe比率'] = performance_df['Sharpe比率'].round(2)
        performance_df['Calmar比率'] = performance_df['Calmar比率'].round(2)
        performance_df['投资胜率'] = performance_df['投资胜率'].apply(lambda x: format(x, '.2%'))
        performance_df['平均损益比'] = performance_df['平均损益比'].round(2)
        # 分月度超额收益
        nav_df['trade_date'] = nav_df.index
        nav_df['trade_dt'] = nav_df['trade_date'].apply(lambda x: datetime.strptime(x, "%Y%m%d"))
        nav_df['month'] = nav_df['trade_dt'].apply(lambda x: x.month)
        nav_df['year'] = nav_df['trade_dt'].apply(lambda x: x.year)
        month_end = nav_df[nav_df['month'].shift(-1) != nav_df['month']]['trade_date'].tolist()

        month_excess = nav_df.reindex(month_end)['超额'].pct_change().dropna()
        month_excess = pd.merge(month_excess, nav_df[['month', 'year']], left_index=True, right_index=True)
        month_excess = pd.pivot_table(month_excess, index='year', columns='month', values='超额').sort_index()
        month_excess = month_excess.T.reindex(np.arange(1, 13)).sort_index().T
        month_excess.columns = [str(x) + '月' for x in month_excess.columns]
        month_excess['全年'] = (1 + month_excess.fillna(0.)).prod(axis=1) - 1
        for i in range(len(month_excess.index)):
            values = month_excess.iloc[i].values
            month_excess.iloc[i, :] = [format(x, '.2%') if x == x else x for x in values]
        # plot
        df = nav_df[nav_df.columns[:3]]
        self.plotly_line(df, "产品净值曲线", "D:\\量化产品跟踪\\深度报告相关\\净值曲线.html", figsize=(1200, 500))

        performance_df = performance_df.T
        performance_df.index.name = "指标名称"
        performance_df = performance_df.reset_index()
        fig = ff.create_table(performance_df)
        fig.layout.autosize = False
        fig.layout.width = 400
        fig.layout.height = 400

        plot_ly(fig, filename="D:\\量化产品跟踪\\深度报告相关\\收益指标统计.html", auto_open=False)

        month_excess.to_csv('D:\\量化产品跟踪\\深度报告相关\\月度超额收益.csv', encoding="gbk")


if __name__ == '__main__':
    nv_series = get_fund_nav_from_sql('20170721', '20220218', {"天演6号": "SN1819"})
    FundNavAnalysor('20170721', '20220218', '000905', nv_series).run()
    # nv_series = pd.read_excel(
    #     "D:\\研究基地\\L-From_路遥\\1-跟踪代销\\天演\\净值文件\\天演资本代表产品净值20220106.xlsx", sheet_name="300指增", header=1, index_col=0)
    # nv_series['trade_date'] = nv_series.index
    # nv_series['trade_date'] = nv_series['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
    # nv_series.rename(columns={"复权净值": "天演300指增"}, inplace=True)
    # nv_series = nv_series.set_index('trade_date')['天演300指增']
    # FundNavAnalysor('20191231', '20211231', '000300', nv_series).run()


"""
期货市场数据
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class FutureMarketIndex:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_future_market_data'

    def get_future_market_data(self):
        """
        Wind商品指数数据
        """
        index_list = ['NMBM.WI', 'NMFI.WI', 'OOFI.WI', 'SOFI.WI', 'NFFI.WI',
                      'JJRI.WI', 'ENFI.WI', 'CIFI.WI', 'CRFI.WI', 'APFI.WI',
                      'CCFI.WI']
        all_data = []
        for index_name in index_list:
            res = w.wsd(index_name, "close,volume,oi_index", self.start_date, self.end_date, "")
            if res.ErrorCode != 0:
                data = pd.DataFrame()
                print("fetch future index data error: start_date = {}, end_date = {}".format(
                    self.start_date, self.end_date))
            else:
                data = pd.DataFrame(res.Data, index=res.Fields, columns=res.Times).T
                data.index.name = 'trade_date'
                data.reset_index(inplace=True)
                data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
                data['index_name'] = index_name

            all_data.append(data)
        all_data = pd.concat(all_data)

        return all_data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_future_market_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_future_market_data(
                id int auto_increment primary key,
                trade_date date not null,
                index_name char(20),
                CLOSE decimal(6, 2),
                VOLUME decimal(9, 0),
                OI_INDEX decimal(9, 0)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_future_market_data()
            WriteToDB().write_to_db(data, self.table_name)


class FuturePrice:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_future_price'

    def get_future_price_data(self):
        """
        常见期货行情数据
        """
        index_list = ['CU.SHF', 'HC.SHF', 'I.DCE', 'J.DCE', 'JM.DCE', 'M.DCE',
                      'MA.CZC', 'P.DCE', 'PP.DCE', 'RB.SHF', 'Y.DCE', 'ZN.SHF']
        res = w.wsd(','.join(index_list), "close", self.start_date, self.end_date, "")
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch future index data error: start_date = {}, end_date = {}".format(
                self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_future_price_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_future_price(
                id int auto_increment primary key,
                trade_date date not null unique,
                CU.SHF decimal(6, 0),
                HC.SHF decimal(6, 0),
                I.DCE decimal(6, 0),
                J.DCE decimal(6, 0),
                JM.DCE decimal(6, 0),
                M.DCE decimal(6, 0),
                MA.CZC decimal(6, 0),
                P.DCE decimal(6, 0),
                PP.SHF decimal(6, 0),
                RB.SHF decimal(6, 0),
                Y.DCE decimal(6, 0),
                ZN.SHF decimal(6, 0)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_future_price_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    FutureMarketIndex('2020-01-01', '2021-06-29', is_increment=1).get_construct_result()
    # FuturePrice('2015-01-05', '2021-06-29', is_increment=0).get_construct_result()


import pandas as pd
from sqlalchemy import create_engine
from datetime import datetime
from hbshare.quant.CChen.func import generate_table
import hbshare as hbs
import pymysql
from hbshare.quant.CChen.cons import (
    db,
    db_tables,
    HSJY_EXCHANGE_INE,
    HSJY_EXCHANGE_CZCE,
    HSJY_EXCHANGE_SHFE,
    HSJY_EXCHANGE_DCE,
    HSJY_EXCHANGE_CFFEX
)

page_size_con = 49999

sql_l = '''(
    `JSID` bigint not null COMMENT \'恒生聚源ID\',
    `TDATE`  date not null COMMENT \'交易日期\',
    `EXCHANGE` int COMMENT \'交易所\',
    `CCODE` int COMMENT \'合约代码-恒生聚源\',
    `PCODE` int COMMENT \'品种代码-恒生聚源\',
    `DYEAR` int COMMENT \'交割年\',
    `DMONTH` int COMMENT \'交割月\',
    `DDATE` date COMMENT \'交割日期\',
    `LDATE` date COMMENT \'最后交易日期\',
    `OPEN` float,
    `HIGH` float,
    `LOW` float,
    `CLOSE` float,
    `SETTLE` float,
    `VOL` int,
    `OI` int,
    `OICHG` int,
    `AMT` double,
    primary key (`JSID`)
    )
    '''

sql_l_info = '''(
    `JSID` bigint not null COMMENT \'恒生聚源ID\',
    `EXCHANGE` int COMMENT \'交易所\',
    `CNAME` varchar(255) COMMENT \'合约名称\',
    `CCODE` bigint COMMENT \'合约代码-恒生聚源\',
    `PCODE` int COMMENT \'品种代码-恒生聚源\',
    `PCODED` bigint COMMENT \'品种唯一代码-恒生聚源\',
    `EDATE` date COMMENT \'上市日期\',
    `DDATE` date COMMENT \'交割日期\',
    `LDATE` date COMMENT \'最后交易日期\',
    `MULT` int comment \'合约乘数\',
    `MULTINFO` varchar(255) COMMENT \'每手合约乘数\',
    `PRICEUNIT` varchar(255) COMMENT \'报价单位\',
    `SLIP` varchar(255) COMMENT \'最小变动价位\',
    `CHGLIMIT`  varchar(255) COMMENT \'涨跌停幅度\',
    primary key (`JSID`)
    )
    '''

sql_l_info_p = '''(
    `JSID` bigint not null COMMENT \'恒生聚源ID\',
    `EXCHANGE` int COMMENT \'交易所\',
    `PNAME` varchar(255) COMMENT \'品种名称\',
    `PCODED` bigint COMMENT \'品种唯一代码-恒生聚源\',
    `PCODE` int COMMENT \'品种代码-恒生聚源\',
    `CODE` varchar(255) COMMENT \'交易所代码\',
    `PTYPE` int COMMENT \'品种合约类型-恒生聚源\',
    primary key (`JSID`)
    )
    '''

sql_l_wr = '''(
    `JSID` bigint not null COMMENT \'恒生聚源ID\',
    `TDATE`  date not null COMMENT \'交易日期\',
    `EXCHANGE` int COMMENT \'交易所\',
    `PCODE` int COMMENT \'品种代码-恒生聚源\',
    `PNAME` varchar(255) COMMENT \'品种名称\',
    `FREQ` int COMMENT \'公布频率\',
    `UNITCODE` int COMMENT \'单位代码\',
    `UNITNAME` varchar(255) COMMENT \'单位名称\',
    `WRQPRIOR` int  COMMENT \'上期仓单量\',
    `WRQINCREASE` int COMMENT \'仓单增加量\',
    `WRQWRITEOFF` int COMMENT \'仓单注销量\',
    `WRQCURRENT` int COMMENT \'本期仓单量\',
    `WRQPREDICTION` int COMMENT \'有效预报量\',
    `STOCKPRIOR` int COMMENT \'上期库存小计\',
    `STOCKCURRENT` int COMMENT \'本期库存小计\',
    `AVAILABLESTOCKPRIOR` int COMMENT \'上期可用库容量\',
    `AVAILABLESTOCKCURRENT` int COMMENT \'本期可用库容量\',
    primary key (`JSID`)
    )
    '''

sql_l_member = '''(
    `JSID` bigint not null COMMENT \'恒生聚源ID\',
    `TDATE`  date not null COMMENT \'交易日期\',
    `EXCHANGE` int COMMENT \'交易所\',
    `CCODE` bigint COMMENT \'合约代码-恒生聚源\',
    `CONTRACT` varchar(255) COMMENT \'交易代码，品种代码+到期日期\',
    `RANKN` int COMMENT \'名次\',
    `MCODE` varchar(255) COMMENT \'会员号\',
    `MNAME` varchar(255) COMMENT \'会员简称\',
    `INDICATORCODE` int COMMENT \'指标代码\',
    `INDICATORNAME` varchar(255) COMMENT \'指标名称\',
    `INDICATORVOL` int COMMENT \'指标数量(手)\',
    `INDICATORCHG` int COMMENT \'较上期增减量(手)\',
    primary key (`JSID`)
    )
    '''


def trans_data(data, contract_info, data_info=None):
    df = pd.DataFrame(data)
    df['TDATE'] = pd.to_datetime(df['ENDDATE']).dt.date
    df['DDATE'] = df['INNERCODE'].apply(
        lambda x: contract_info[
            contract_info['CCode'.upper()] == x
            ].reset_index(drop=True)['DDATE'][0]
    )
    df['DDATE'] = pd.to_datetime(df['DDATE']).dt.date
    df['LDATE'] = df['INNERCODE'].apply(
        lambda x: contract_info[
            contract_info['CCode'.upper()] == x
            ].reset_index(drop=True)['LDate'.upper()][0]
    )
    df['LDATE'] = pd.to_datetime(df['LDATE']).dt.date
    # df['PRODUCT'] = df['OPTIONCODE'].apply(
    #     lambda x: data_info[data_info['PRODUCTCODE'] == str(x)].reset_index(drop=True)['PRODUCTNAME'][0]
    # )
    df = df.rename(
        columns={
            'CONTRACTNAME': 'NAME',
            'SETTLEMENTYEAR': 'DYEAR',
            'SETTLEMENTMONTH': 'DMONTH',
            'OPENPRICE': 'OPEN',
            'HIGHPRICE': 'HIGH',
            'LOWPRICE': 'LOW',
            'CLOSEPRICE': 'CLOSE',
            'SETTLEPRICE': 'SETTLE',
            'PREVSETTLEPRICE': 'PRESETTLE',
            'VOLUME': 'VOL',
            'OPENINTEREST': 'OI',
            'OPENINTERESTCHANGE': 'OICHG',
            'TURNOVER': 'AMT',
            'OPTIONCODE': 'PCODE',
            'INNERCODE': 'CCODE'
        }
    )[
        [
            'TDATE',
            'EXCHANGE',
            # 'NAME',
            'CCODE',
            'PCODE',
            'DYEAR',
            'DMONTH',
            'DDATE',
            'LDATE',
            # 'PRESETTLE',
            'OPEN',
            'HIGH',
            'LOW',
            'CLOSE',
            'SETTLE',
            'VOL',
            'OI',
            'OICHG',
            'AMT',
            'JSID'
        ]
    ]
    return df


def hsjy_fut_wr(db_path, sql_info, exchanges=None, table='hsjy_fut_wr', page_size=None):
    if page_size is None:
        page_size = page_size_con

    if exchanges is None:
        exchanges = [HSJY_EXCHANGE_SHFE, HSJY_EXCHANGE_DCE, HSJY_EXCHANGE_CZCE, HSJY_EXCHANGE_INE, HSJY_EXCHANGE_CFFEX]

    generate_table(
        database='daily_data',
        table=table,
        generate_sql=sql_l_wr,
        sql_ip=sql_info['ip'],
        sql_user=sql_info['user'],
        sql_pass=sql_info['pass'],
        table_comment='from ' + db_tables['hsjy_wr']
    )
    print(table + ' generated')

    latest_jsid_in_db = pd.read_sql_query(
        'select distinct `JSID` from ' + table + ' order by `JSID` desc limit 1', create_engine(db_path)
    )
    if len(latest_jsid_in_db) > 0:
        latest_jsid = latest_jsid_in_db['JSID'][0]
    else:
        latest_jsid = 0

    print('\t' + 'Latest jsid in db: ' + str(latest_jsid))

    if len(exchanges) > 1:
        ex_sql = 'EXCHANGE in ' + str(tuple(exchanges))
    elif len(exchanges) == 1:
        ex_sql = 'EXCHANGE=\'' + exchanges[0] + '\''
    else:
        raise ValueError('No exchange')

    while 1:
        sql = (
                'select * from (select * from ' + db_tables['hsjy_wr']
                + ' where JSID>' + str(latest_jsid)
                + ' and ' + ex_sql.replace('EXCHANGE', 'EXCHANGECODE')
                + ' order by JSID) where rownum<=' + str(page_size)
        ).replace(
            '*', 'JSID, ENDDATE, EXCHANGECODE, REPORTPERIOD, OPTIONCODE, OPTIONNAME, UNITCODE, UNITNAME, '
                 'WRQPrior, WRQIncrease, WRQWriteOff, WRQCurrent, WRQPrediction, StockPrior, StockCurrent, '
                 'AvailableStockPrior, AvailableStockCurrent'.upper()
        )

        wr_info = hbs.db_data_query(
            db=db,
            sql=sql,
            page_size=page_size
        )
        df = pd.DataFrame(wr_info['data']).rename(
            columns={
                'ENDDATE': 'TDATE',
                'EXCHANGECODE': 'EXCHANGE',
                'REPORTPERIOD': 'FREQ',
                'OPTIONCODE': 'PCODE',
                'OPTIONNAME': 'PNAME',
            }
        )
        if len(df) == 0:
            print('\t\t' + 'No more new info data')
            return
        col = df.columns.tolist()
        col.pop(col.index('ROW_ID'))
        df[col].to_sql(table, create_engine(db_path), if_exists='append', index=False)
        latest_jsid = df['JSID'].tolist()[-1]
        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))


def hsjy_fut_com_info(db_path, sql_info, exchanges=None, table='hsjy_fut_info_c', page_size=None):
    if page_size is None:
        page_size = page_size_con

    if exchanges is None:
        exchanges = [HSJY_EXCHANGE_SHFE, HSJY_EXCHANGE_DCE, HSJY_EXCHANGE_CZCE, HSJY_EXCHANGE_INE, HSJY_EXCHANGE_CFFEX]

    generate_table(
        database='daily_data',
        table=table,
        generate_sql=sql_l_info,
        sql_ip=sql_info['ip'],
        sql_user=sql_info['user'],
        sql_pass=sql_info['pass'],
        table_comment='from ' + db_tables['hsjy_fut_contract_info']
    )
    print(table + ' generated')

    latest_jsid_in_db = pd.read_sql_query(
        'select distinct `JSID` from ' + table + ' order by `JSID` desc limit 1', create_engine(db_path)
    )
    if len(latest_jsid_in_db) > 0:
        latest_jsid = latest_jsid_in_db['JSID'][0]
    else:
        latest_jsid = 0

    print('\t' + 'Latest jsid in db: ' + str(latest_jsid))

    if len(exchanges) > 1:
        ex_sql = 'EXCHANGE in ' + str(tuple(exchanges))
    elif len(exchanges) == 1:
        ex_sql = 'EXCHANGE=\'' + exchanges[0] + '\''
    else:
        raise ValueError('No exchange')

    while 1:
        sql = (
                'select * from (select * from ' + db_tables['hsjy_fut_contract_info']
                + ' where JSID>' + str(latest_jsid)
                + ' and IFREAL=1 and ' + ex_sql.replace('EXCHANGE', 'EXCHANGECODE')
                + ' order by JSID) where rownum<=' + str(page_size)
        ).replace(
            '*', 'JSID, CONTRACTINNERCODE, CONTRACTNAME, EXCHANGECODE, OPTIONCODE, LASTTRADINGDATE, DELIVERYDATE, '
                 'EffectiveDate, CMValue, ContractMultiplier, '
                 'PriceUnit, LittlestChangeUnit, ChangePCTLimit, VarietyInnerCode'.upper()
        )

        contract_info = hbs.db_data_query(
            db=db,
            sql=sql,
            page_size=page_size
        )
        df = pd.DataFrame(contract_info['data']).rename(
            columns={
                'CONTRACTINNERCODE': 'CCODE',
                'CONTRACTNAME': 'CNAME',
                'EXCHANGECODE': 'EXCHANGE',
                'OPTIONCODE': 'PCODE',
                'VarietyInnerCode'.upper(): 'PCODED',
                'EffectiveDate'.upper(): 'EDATE',
                'LASTTRADINGDATE': 'LDATE',
                'DELIVERYDATE': 'DDATE',
                'CMVALUE': 'MULT',
                'ContractMultiplier'.upper(): 'MULTINFO',
                'LittlestChangeUnit'.upper(): 'SLIP',
                'ChangePCTLimit'.upper(): 'CHGLIMIT',
            }
        )
        if len(df) == 0:
            print('\t\t' + 'No more new info data')
            return
        df[
            [
                'JSID', 'CCODE', 'CNAME', 'EXCHANGE', 'PCODE', 'PCODED', 'EDATE', 'LDATE', 'DDATE',
                'MULT', 'MULTINFO', 'SLIP', 'CHGLIMIT', 'PRICEUNIT'
            ]
        ].to_sql(table, create_engine(db_path), if_exists='append', index=False)
        latest_jsid = df['JSID'].tolist()[-1]
        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))


def hsjy_fut_pro_info(db_path, sql_info, exchanges=None, table='hsjy_fut_info_p', page_size=None):
    if page_size is None:
        page_size = page_size_con

    if exchanges is None:
        exchanges = [HSJY_EXCHANGE_SHFE, HSJY_EXCHANGE_DCE, HSJY_EXCHANGE_CZCE, HSJY_EXCHANGE_INE, HSJY_EXCHANGE_CFFEX]

    generate_table(
        database='daily_data',
        table=table,
        generate_sql=sql_l_info_p,
        sql_ip=sql_info['ip'],
        sql_user=sql_info['user'],
        sql_pass=sql_info['pass'],
        table_comment='from ' + db_tables['hsjy_product_info']
    )
    print(table + ' generated')

    latest_jsid_in_db = pd.read_sql_query(
        'select distinct `JSID` from ' + table + ' order by `JSID` desc limit 1', create_engine(db_path)
    )
    if len(latest_jsid_in_db) > 0:
        latest_jsid = latest_jsid_in_db['JSID'][0]
    else:
        latest_jsid = 0

    print('\t' + 'Latest jsid in db: ' + str(latest_jsid))

    if len(exchanges) > 1:
        ex_sql = 'EXCHANGE in ' + str(tuple(exchanges))
    elif len(exchanges) == 1:
        ex_sql = 'EXCHANGE=\'' + exchanges[0] + '\''
    else:
        raise ValueError('No exchange')

    while 1:
        sql = (
                'select * from (select * from ' + db_tables['hsjy_product_info']
                + ' where JSID>' + str(latest_jsid)
                + ' and ' + ex_sql + ' order by JSID) where rownum<=' + str(page_size)
        ).replace(
            '*', 'JSID, CONTRACTINNERCODE, CONTRACTNAME, EXCHANGE, TradingCode, ContractType, ContractOption'.upper()
        )

        contract_info = hbs.db_data_query(
            db=db,
            sql=sql,
            page_size=page_size
        )
        df = pd.DataFrame(contract_info['data']).rename(
            columns={
                'CONTRACTINNERCODE': 'PCODED',
                'CONTRACTNAME': 'PNAME',
                'ContractOption'.upper(): 'PCODE',
                'TradingCode'.upper(): 'CODE',
                'ContractType'.upper(): 'PTYPE'
            }
        )
        if len(df) == 0:
            print('\t\t' + 'No more new info data')
            return
        df[
            [
                'JSID', 'EXCHANGE', 'PCODE', 'PNAME', 'PCODED', 'CODE', 'PTYPE'
            ]
        ].to_sql(table, create_engine(db_path), if_exists='append', index=False)
        latest_jsid = df['JSID'].tolist()[-1]
        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))


def hsjy_fut_com(
        db_path, sql_info, exchanges=None, table='hsjy_fut_com', table_info='hsjy_fut_info_c', page_size=None
):
    if page_size is None:
        page_size = page_size_con

    if exchanges is None:
        exchanges = [HSJY_EXCHANGE_SHFE, HSJY_EXCHANGE_DCE, HSJY_EXCHANGE_CZCE, HSJY_EXCHANGE_INE]

    generate_table(
        database='daily_data',
        table=table,
        generate_sql=sql_l,
        sql_ip=sql_info['ip'],
        sql_user=sql_info['user'],
        sql_pass=sql_info['pass'],
        table_comment='from ' + db_tables['hsjy_com_daily_quote']
    )
    print(table + ' generated')

    latest_jsid_in_db = pd.read_sql_query(
        'select distinct `JSID` from ' + table + ' order by `JSID` desc limit 1', create_engine(db_path)
    )
    if len(latest_jsid_in_db) > 0:
        latest_jsid = latest_jsid_in_db['JSID'][0]
    else:
        latest_jsid = 0

    print('\tLatest jsid in db: ' + str(latest_jsid))

    if len(exchanges) > 1:
        ex_sql = 'EXCHANGE in ' + str(tuple(exchanges))
    elif len(exchanges) == 1:
        ex_sql = 'EXCHANGE=\'' + exchanges[0] + '\''
    else:
        raise ValueError('No exchange')

    contract_info = pd.read_sql_query('select * from ' + table_info, create_engine(db_path))

    while 1:
        sql = (
                'select * from (select * from ' + db_tables['hsjy_com_daily_quote']
                + ' where  ENDDATE>=TRUNC(to_date(' + datetime(1999, 12, 31).strftime('%Y%m%d')
                + ', \'yyyymmdd\')) and JSID>' + str(latest_jsid)
                + ' and ' + ex_sql + ' order by JSID) where rownum<=' + str(page_size)
        )

        data = hbs.db_data_query(db=db, sql=sql, page_size=page_size)
        if len(data['data']) == 0:
            print('\t\t' + 'No more new quote data')
            return

        df = trans_data(
            data=data['data'], contract_info=contract_info,
            # data_info=df_info
        )
        df.to_sql(table, create_engine(db_path), if_exists='append', index=False)
        latest_jsid = df['JSID'].tolist()[-1]
        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))


def hsjy_fut_member(db_path, sql_info, table='hsjy_fut_memberrank', page_size=None):
    if page_size is None:
        page_size = page_size_con

    generate_table(
        database='daily_data',
        table=table,
        generate_sql=sql_l_member,
        sql_ip=sql_info['ip'],
        sql_user=sql_info['user'],
        sql_pass=sql_info['pass'],
        table_comment='from ' + db_tables['hsjy_member_rank']
    )
    print(table + ' generated')

    latest_jsid_in_db = pd.read_sql_query(
        'select distinct `JSID` from ' + table + ' order by `JSID` desc limit 1', create_engine(db_path)
    )
    if len(latest_jsid_in_db) > 0:
        latest_jsid = latest_jsid_in_db['JSID'][0]
    else:
        latest_jsid = 0

    print('\tLatest jsid in db: ' + str(latest_jsid))

    while 1:
        member_sql = (
                'select * from (select * from ' + db_tables['hsjy_member_rank']
                + ' where JSID>' + str(latest_jsid)
                + ' order by JSID) where rownum<=' + str(page_size)
        ).replace(
            '*', 'JSID, ENDDATE, EXCHANGECODE, ContractInnerCode, ContractCode, RankNumber, MemberCode, MemberAbbr, '
                 'IndicatorCode, IndicatorName, IndicatorVolume, ChangeVolume'.upper()
        )
        member_rank_info = hbs.db_data_query(
            db=db,
            sql=member_sql,
            page_size=page_size
        )
        df = pd.DataFrame(member_rank_info['data']).rename(
            columns={
                'ENDDATE': 'TDATE',
                'EXCHANGECODE': 'EXCHANGE',
                'ContractInnerCode'.upper(): 'CCODE',
                'ContractCode'.upper(): 'CONTRACT',
                'RankNumber'.upper(): 'RANKN',
                'MemberCode'.upper(): 'MCODE',
                'MemberAbbr'.upper(): 'MNAME',
                'IndicatorVolume'.upper(): 'INDICATORVOL',
                'ChangeVolume'.upper(): 'INDICATORCHG'
            }
        )
        if len(df) == 0:
            print('\t\t' + 'No more new member rank data')
            return
        col = df.columns.tolist()
        col.pop(col.index('ROW_ID'))
        df[col].to_sql(table, create_engine(db_path), if_exists='append', index=False)
        latest_jsid = df['JSID'].tolist()[-1]
        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))

